@@ -11,11 +11,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from keystoneclient import access
-from keystoneclient import auth
-from keystoneclient.auth.identity import access as access_plugin
-from keystoneclient.auth.identity import v3
-from keystoneclient.auth import token_endpoint
+from keystoneauth1 import access
+from keystoneauth1.identity import access as access_plugin
+from keystoneauth1.identity import generic
+from keystoneauth1 import loading as ks_loading
+from keystoneauth1 import session
+from keystoneauth1 import token_endpoint
 from oslo_config import cfg
 from oslo_context import context
 from oslo_log import log as logging
@@ -24,6 +25,7 @@ from oslo_middleware import request_id as oslo_request_id
 from oslo_utils import importutils
 import six
 
+from heat.common import config
 from heat.common import endpoint_utils
 from heat.common import exception
 from heat.common.i18n import _LE, _LW
@@ -37,25 +39,37 @@ LOG = logging.getLogger(__name__)
 
 # Note, we yield the options via list_opts to enable generation of the
 # sample heat.conf, but we don't register these options directly via
-# cfg.CONF.register*, it's done via auth.register_conf_options
-# Note, only auth_plugin = v3password is expected to work, example config:
+# cfg.CONF.register*, it's done via ks_loading.register_auth_conf_options
+# Note, only auth_type = v3password is expected to work, example config:
 # [trustee]
-# auth_plugin = password
+# auth_type = password
 # auth_url = http://192.168.1.2:35357
 # username = heat
 # password = password
 # user_domain_id = default
-V3_PASSWORD_PLUGIN = 'v3password'
+PASSWORD_PLUGIN = 'password'
 TRUSTEE_CONF_GROUP = 'trustee'
-auth.register_conf_options(cfg.CONF, TRUSTEE_CONF_GROUP)
+ks_loading.register_auth_conf_options(cfg.CONF, TRUSTEE_CONF_GROUP)
 
 
 def list_opts():
-    trustee_opts = auth.conf.get_common_conf_options()
-    trustee_opts.extend(auth.conf.get_plugin_options(V3_PASSWORD_PLUGIN))
+    trustee_opts = ks_loading.get_auth_common_conf_options()
+    trustee_opts.extend(ks_loading.get_auth_plugin_conf_options(
+        PASSWORD_PLUGIN))
     yield TRUSTEE_CONF_GROUP, trustee_opts
 
 
+def _moved_attr(new_name):
+
+    def getter(self):
+        return getattr(self, new_name)
+
+    def setter(self, value):
+        setattr(self, new_name, value)
+
+    return property(getter, setter)
+
+
 class RequestContext(context.RequestContext):
     """Stores information about the security context.
 
@@ -63,42 +77,42 @@ class RequestContext(context.RequestContext):
     additional request information.
     """
 
-    def __init__(self, auth_token=None, username=None, password=None,
-                 aws_creds=None, tenant=None, user_id=None,
-                 tenant_id=None, auth_url=None, roles=None, is_admin=None,
-                 read_only=False, show_deleted=False,
-                 overwrite=True, trust_id=None, trustor_user_id=None,
-                 request_id=None, auth_token_info=None, region_name=None,
-                 auth_plugin=None, trusts_auth_plugin=None,
-                 user_domain_id=None, project_domain_id=None, **kwargs):
+    def __init__(self, username=None, password=None, aws_creds=None,
+                 auth_url=None, roles=None, is_admin=None, read_only=False,
+                 show_deleted=False, overwrite=True, trust_id=None,
+                 trustor_user_id=None, request_id=None, auth_token_info=None,
+                 region_name=None, auth_plugin=None, trusts_auth_plugin=None,
+                 user_domain_id=None, project_domain_id=None,
+                 project_name=None, **kwargs):
         """Initialisation of the request context.
 
         :param overwrite: Set to False to ensure that the greenthread local
             copy of the index is not overwritten.
-
-         :param kwargs: Extra arguments that might be present, but we ignore
-            because they possibly came in from older rpc messages.
         """
-        super(RequestContext, self).__init__(auth_token=auth_token,
-                                             user=username, tenant=tenant,
-                                             is_admin=is_admin,
+        if user_domain_id:
+            kwargs['user_domain'] = user_domain_id
+        if project_domain_id:
+            kwargs['project_domain'] = project_domain_id
+
+        super(RequestContext, self).__init__(is_admin=is_admin,
                                              read_only=read_only,
                                              show_deleted=show_deleted,
                                              request_id=request_id,
-                                             user_domain=user_domain_id,
-                                             project_domain=project_domain_id)
+                                             roles=roles,
+                                             overwrite=overwrite,
+                                             **kwargs)
 
         self.username = username
-        self.user_id = user_id
         self.password = password
         self.region_name = region_name
         self.aws_creds = aws_creds
-        self.tenant_id = tenant_id
+        self.project_name = project_name
         self.auth_token_info = auth_token_info
         self.auth_url = auth_url
-        self.roles = roles or []
         self._session = None
         self._clients = None
+        self._keystone_session = session.Session(
+            **config.get_ssl_options('keystone'))
         self.trust_id = trust_id
         self.trustor_user_id = trustor_user_id
         self.policy = policy.Enforcer()
@@ -110,18 +124,46 @@ class RequestContext(context.RequestContext):
         else:
             self.is_admin = is_admin
 
+        # context scoped cache dict where the key is a class of the type of
+        # object being cached and the value is the cache implementation class
+        self._object_cache = {}
+
+    def cache(self, cache_cls):
+        cache = self._object_cache.get(cache_cls)
+        if not cache:
+            cache = cache_cls()
+            self._object_cache[cache_cls] = cache
+        return cache
+
+    user_id = _moved_attr('user')
+    tenant_id = _moved_attr('tenant')
+
     @property
     def session(self):
         if self._session is None:
             self._session = db_api.get_session()
         return self._session
 
+    @property
+    def keystone_session(self):
+        if self.auth_needs_refresh():
+            self.reload_auth_plugin()
+            self.clients.invalidate_plugins()
+        self._keystone_session.auth = self.auth_plugin
+        return self._keystone_session
+
     @property
     def clients(self):
         if self._clients is None:
             self._clients = clients.Clients(self)
         return self._clients
 
+    def auth_needs_refresh(self):
+        auth_ref = self.auth_plugin.get_auth_ref(self._keystone_session)
+        return (cfg.CONF.reauthentication_auth_method == 'trusts'
+                and auth_ref.will_expire_soon(
+                    cfg.CONF.stale_token_duration))
+
     def to_dict(self):
         user_idt = '{user} {tenant}'.format(user=self.user_id or '-',
                                             tenant=self.tenant_id or '-')
@@ -131,7 +173,7 @@ class RequestContext(context.RequestContext):
                 'user_id': self.user_id,
                 'password': self.password,
                 'aws_creds': self.aws_creds,
-                'tenant': self.tenant,
+                'tenant': self.project_name,
                 'tenant_id': self.tenant_id,
                 'trust_id': self.trust_id,
                 'trustor_user_id': self.trustor_user_id,
@@ -139,7 +181,7 @@ class RequestContext(context.RequestContext):
                 'auth_url': self.auth_url,
                 'roles': self.roles,
                 'is_admin': self.is_admin,
-                'user': self.user,
+                'user': self.username,
                 'request_id': self.request_id,
                 'show_deleted': self.show_deleted,
                 'region_name': self.region_name,
@@ -149,7 +191,40 @@ class RequestContext(context.RequestContext):
 
     @classmethod
     def from_dict(cls, values):
-        return cls(**values)
+        return cls(
+            auth_token=values.get('auth_token'),
+            username=values.get('username'),
+            user=values.get('user_id'),
+            password=values.get('password'),
+            aws_creds=values.get('aws_creds'),
+            project_name=values.get('tenant'),
+            tenant=values.get('tenant_id'),
+            trust_id=values.get('trust_id'),
+            trustor_user_id=values.get('trustor_user_id'),
+            auth_token_info=values.get('auth_token_info'),
+            auth_url=values.get('auth_url'),
+            roles=values.get('roles'),
+            is_admin=values.get('is_admin'),
+            request_id=values.get('request_id'),
+            show_deleted=values.get('show_deleted', False),
+            region_name=values.get('region_name'),
+            user_domain_id=values.get('user_domain'),
+            project_domain_id=values.get('project_domain')
+        )
+
+    def to_policy_values(self):
+        policy = super(RequestContext, self).to_policy_values()
+
+        # NOTE(jamielennox): These are deprecated values passed to oslo.policy
+        # for enforcement. They shouldn't be needed as the base class defines
+        # what should be used when writing policy but are maintained for
+        # compatibility.
+        policy['user'] = self.user_id
+        policy['tenant'] = self.tenant_id
+        policy['is_admin'] = self.is_admin
+        policy['auth_token_info'] = self.auth_token_info
+
+        return policy
 
     @property
     def keystone_v3_endpoint(self):
@@ -160,9 +235,9 @@ class RequestContext(context.RequestContext):
             if auth_uri:
                 return auth_uri
             else:
-                LOG.error('Keystone API endpoint not provided. Set '
-                          'auth_uri in section [clients_keystone] '
-                          'of the configuration file.')
+                LOG.error(_LE('Keystone API endpoint not provided. Set '
+                              'auth_uri in section [clients_keystone] '
+                              'of the configuration file.'))
                 raise exception.AuthorizationFailure()
 
     @property
@@ -170,7 +245,7 @@ class RequestContext(context.RequestContext):
         if self._trusts_auth_plugin:
             return self._trusts_auth_plugin
 
-        self._trusts_auth_plugin = auth.load_from_conf_options(
+        self._trusts_auth_plugin = ks_loading.load_auth_from_conf_options(
             cfg.CONF, TRUSTEE_CONF_GROUP, trust_id=self.trust_id)
 
         if self._trusts_auth_plugin:
@@ -188,7 +263,7 @@ class RequestContext(context.RequestContext):
         if 'user_domain_id' in cfg.CONF.keystone_authtoken:
             trustee_user_domain = cfg.CONF.keystone_authtoken.user_domain_id
 
-        self._trusts_auth_plugin = v3.Password(
+        self._trusts_auth_plugin = generic.Password(
             username=cfg.CONF.keystone_authtoken.admin_user,
             password=cfg.CONF.keystone_authtoken.admin_password,
             user_domain_id=trustee_user_domain,
@@ -198,11 +273,10 @@ class RequestContext(context.RequestContext):
 
     def _create_auth_plugin(self):
         if self.auth_token_info:
-            auth_ref = access.AccessInfo.factory(body=self.auth_token_info,
-                                                 auth_token=self.auth_token)
+            access_info = access.create(body=self.auth_token_info,
+                                        auth_token=self.auth_token)
             return access_plugin.AccessInfoPlugin(
-                auth_url=self.keystone_v3_endpoint,
-                auth_ref=auth_ref)
+                auth_ref=access_info, auth_url=self.keystone_v3_endpoint)
 
         if self.auth_token:
             # FIXME(jamielennox): This is broken but consistent. If you
@@ -213,13 +287,13 @@ class RequestContext(context.RequestContext):
                                         token=self.auth_token)
 
         if self.password:
-            return v3.Password(username=self.username,
-                               password=self.password,
-                               project_id=self.tenant_id,
-                               user_domain_id=self.user_domain,
-                               auth_url=self.keystone_v3_endpoint)
+            return generic.Password(username=self.username,
+                                    password=self.password,
+                                    project_id=self.tenant_id,
+                                    user_domain_id=self.user_domain,
+                                    auth_url=self.keystone_v3_endpoint)
 
-        LOG.error(_LE("Keystone v3 API connection failed, no password "
+        LOG.error(_LE("Keystone API connection failed, no password "
                       "trust or auth_token!"))
         raise exception.AuthorizationFailure()
 
@@ -237,6 +311,46 @@ class RequestContext(context.RequestContext):
         return self._auth_plugin
 
 
+class StoredContext(RequestContext):
+    def _load_keystone_data(self):
+        self._keystone_loaded = True
+        auth_ref = self.clients.client('keystone').auth_ref
+
+        self.roles = auth_ref.role_names
+        self.user_domain = auth_ref.user_domain_id
+        self.project_domain = auth_ref.project_domain_id
+
+    @property
+    def roles(self):
+        if not getattr(self, '_keystone_loaded', False):
+            self._load_keystone_data()
+        return self._roles
+
+    @roles.setter
+    def roles(self, roles):
+        self._roles = roles
+
+    @property
+    def user_domain(self):
+        if not getattr(self, '_keystone_loaded', False):
+            self._load_keystone_data()
+        return self._user_domain
+
+    @user_domain.setter
+    def user_domain(self, user_domain):
+        self._user_domain = user_domain
+
+    @property
+    def project_domain(self):
+        if not getattr(self, '_keystone_loaded', False):
+            self._load_keystone_data()
+        return self._project_domain
+
+    @project_domain.setter
+    def project_domain(self, project_domain):
+        self._project_domain = project_domain
+
+
 def get_admin_context(show_deleted=False):
     return RequestContext(is_admin=True, show_deleted=show_deleted)
 
@@ -251,10 +365,6 @@ class ContextMiddleware(wsgi.Middleware):
 
         super(ContextMiddleware, self).__init__(app)
 
-    def make_context(self, *args, **kwargs):
-        """Create a context with the given arguments."""
-        return self.ctxcls(*args, **kwargs)
-
     def process_request(self, req):
         """Constructs an appropriate context from extracted auth information.
 
@@ -264,51 +374,36 @@ class ContextMiddleware(wsgi.Middleware):
         headers = req.headers
         environ = req.environ
 
-        try:
-            username = None
-            password = None
-            aws_creds = None
+        username = None
+        password = None
+        aws_creds = None
 
-            if headers.get('X-Auth-User') is not None:
-                username = headers.get('X-Auth-User')
-                password = headers.get('X-Auth-Key')
-            elif headers.get('X-Auth-EC2-Creds') is not None:
-                aws_creds = headers.get('X-Auth-EC2-Creds')
+        if headers.get('X-Auth-User') is not None:
+            username = headers.get('X-Auth-User')
+            password = headers.get('X-Auth-Key')
+        elif headers.get('X-Auth-EC2-Creds') is not None:
+            aws_creds = headers.get('X-Auth-EC2-Creds')
 
-            user_id = headers.get('X-User-Id')
-            user_domain_id = headers.get('X_User_Domain_Id')
-            token = headers.get('X-Auth-Token')
-            tenant = headers.get('X-Project-Name')
-            tenant_id = headers.get('X-Project-Id')
-            project_domain_id = headers.get('X_Project_Domain_Id')
-            region_name = headers.get('X-Region-Name')
-            auth_url = headers.get('X-Auth-Url')
+        project_name = headers.get('X-Project-Name')
+        region_name = headers.get('X-Region-Name')
+        auth_url = headers.get('X-Auth-Url')
 
-            roles = headers.get('X-Roles')
-            if roles is not None:
-                roles = roles.split(',')
-            token_info = environ.get('keystone.token_info')
-            auth_plugin = environ.get('keystone.token_auth')
-            req_id = environ.get(oslo_request_id.ENV_REQUEST_ID)
+        token_info = environ.get('keystone.token_info')
+        auth_plugin = environ.get('keystone.token_auth')
+        req_id = environ.get(oslo_request_id.ENV_REQUEST_ID)
 
-        except Exception:
-            raise exception.NotAuthenticated()
-
-        req.context = self.make_context(
-            auth_token=token,
-            tenant=tenant, tenant_id=tenant_id,
+        req.context = self.ctxcls.from_environ(
+            environ,
+            project_name=project_name,
             aws_creds=aws_creds,
             username=username,
-            user_id=user_id,
             password=password,
             auth_url=auth_url,
-            roles=roles,
             request_id=req_id,
             auth_token_info=token_info,
             region_name=region_name,
             auth_plugin=auth_plugin,
-            user_domain_id=user_domain_id,
-            project_domain_id=project_domain_id)
+        )
 
 
 def ContextMiddleware_filter_factory(global_conf, **local_conf):
