@@ -15,6 +15,7 @@ from glanceclient import client as gc
 from glanceclient import exc
 from glanceclient.openstack.common.apiclient import exceptions
 
+from heat.common.i18n import _
 from heat.engine.clients import client_plugin
 from heat.engine.clients import os as os_client
 from heat.engine import constraints
@@ -28,38 +29,35 @@ class GlanceClientPlugin(client_plugin.ClientPlugin):
 
     service_types = [IMAGE] = ['image']
 
-    def _create(self):
+    supported_versions = [V1, V2] = ['1', '2']
 
+    default_version = V1
+
+    def _create(self, version=None):
         con = self.context
-        endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
-        endpoint = self.url_for(service_type=self.IMAGE,
-                                endpoint_type=endpoint_type)
-        args = {
-            'auth_url': con.auth_url,
-            'service_type': self.IMAGE,
-            'project_id': con.tenant_id,
-            'token': self.auth_token,
-            'endpoint_type': endpoint_type,
-            'cacert': self._get_client_option(CLIENT_NAME, 'ca_file'),
-            'cert_file': self._get_client_option(CLIENT_NAME, 'cert_file'),
-            'key_file': self._get_client_option(CLIENT_NAME, 'key_file'),
-            'insecure': self._get_client_option(CLIENT_NAME, 'insecure')
-        }
+        interface = self._get_client_option(CLIENT_NAME, 'endpoint_type')
 
-        return gc.Client('1', endpoint, **args)
+        return gc.Client(version, session=con.keystone_session,
+                         interface=interface,
+                         service_type=self.IMAGE,
+                         region_name=self._get_region_name())
 
     def _find_with_attr(self, entity, **kwargs):
         """Find a item for entity with attributes matching ``**kwargs``."""
         matches = list(self._findall_with_attr(entity, **kwargs))
         num_matches = len(matches)
         if num_matches == 0:
-            msg = ("No %(name)s matching %(args)s.") % {
+            msg = _("No %(name)s matching %(args)s.") % {
                 'name': entity,
                 'args': kwargs
             }
             raise exceptions.NotFound(msg)
         elif num_matches > 1:
-            raise exceptions.NoUniqueMatch()
+            msg = _("No %(name)s unique match found for %(args)s.") % {
+                'name': entity,
+                'args': kwargs
+            }
+            raise exceptions.NoUniqueMatch(msg)
         else:
             return matches[0]
 
