@@ -12,14 +12,15 @@
 #    under the License.
 
 import collections
-import fnmatch
 import glob
 import itertools
 import os.path
 import re
+import weakref
 
 from oslo_config import cfg
 from oslo_log import log
+from oslo_utils import fnmatch
 import six
 
 from heat.common import environment_format as env_fmt
@@ -113,13 +114,19 @@ class ResourceInfo(object):
         else:
             return MapResourceInfo(registry, path, value)
 
+    __slots__ = ('_registry', 'path', 'name', 'value', 'user_resource')
+
     def __init__(self, registry, path, value):
-        self.registry = registry
+        self._registry = weakref.ref(registry)
         self.path = path
         self.name = path[-1]
         self.value = value
         self.user_resource = True
 
+    @property
+    def registry(self):
+        return self._registry()
+
     def __eq__(self, other):
         if other is None:
             return False
@@ -164,6 +171,8 @@ class ClassResourceInfo(ResourceInfo):
     """Store the mapping of resource name to python class implementation."""
     description = 'Plugin'
 
+    __slots__ = tuple()
+
     def get_class(self, files=None):
         return self.value
 
@@ -172,6 +181,8 @@ class TemplateResourceInfo(ResourceInfo):
     """Store the info needed to start a TemplateResource."""
     description = 'Template'
 
+    __slots__ = ('template_name',)
+
     def __init__(self, registry, path, value):
         super(TemplateResourceInfo, self).__init__(registry, path, value)
         if self.name.endswith(('.yaml', '.template')):
@@ -192,9 +203,10 @@ class TemplateResourceInfo(ResourceInfo):
             data = template_resource.TemplateResource.get_template_file(
                 self.template_name,
                 allowed_schemes)
-        env = self.registry.environment
+        param_defaults = self.registry.param_defaults
         return template_resource.generate_class_from_template(str(self.name),
-                                                              data, env)
+                                                              data,
+                                                              param_defaults)
 
     def get_class_to_instantiate(self):
         from heat.engine.resources import template_resource
@@ -208,6 +220,8 @@ class MapResourceInfo(ResourceInfo):
     """
     description = 'Mapping'
 
+    __slots__ = tuple()
+
     def get_class(self, files=None):
         return None
 
@@ -227,6 +241,8 @@ class GlobResourceInfo(MapResourceInfo):
     """
     description = 'Wildcard Mapping'
 
+    __slots__ = tuple()
+
     def get_resource_info(self, resource_type=None, resource_name=None):
         # NOTE(pas-ha) we end up here only when self.name already
         # ends with * so truncate it
@@ -248,10 +264,10 @@ class GlobResourceInfo(MapResourceInfo):
 class ResourceRegistry(object):
     """By looking at the environment, find the resource implementation."""
 
-    def __init__(self, global_registry, env):
+    def __init__(self, global_registry, param_defaults):
         self._registry = {'resources': {}}
         self.global_registry = global_registry
-        self.environment = env
+        self.param_defaults = param_defaults
 
     def load(self, json_snippet):
         self._load_registry([], json_snippet)
@@ -300,8 +316,8 @@ class ResourceRegistry(object):
         if info is None:
             if name.endswith('*'):
                 # delete all matching entries.
-                for res_name in list(six.iterkeys(registry)):
-                    if (isinstance(registry[res_name], ResourceInfo) and
+                for res_name, reg_info in list(registry.items()):
+                    if (isinstance(reg_info, ResourceInfo) and
                             res_name.startswith(name[:-1])):
                         LOG.warning(_LW('Removing %(item)s from %(path)s'), {
                             'item': res_name,
@@ -393,7 +409,7 @@ class ResourceRegistry(object):
     def matches_hook(self, resource_name, hook):
         """Return whether a resource have a hook set in the environment.
 
-        For a given resource and a hook type, we check to see if the the passed
+        For a given resource and a hook type, we check to see if the passed
         group of resources has the right hook associated with the name.
 
         Hooks are set in this format via `resources`:
@@ -463,7 +479,7 @@ class ResourceRegistry(object):
         # handle: "OS::*" -> "Dreamhost::*"
         def is_a_glob(resource_type):
             return resource_type.endswith('*')
-        globs = six.moves.filter(is_a_glob, six.iterkeys(self._registry))
+        globs = six.moves.filter(is_a_glob, iter(self._registry))
         for pattern in globs:
             if self._registry[pattern].matches(resource_type):
                 yield self._registry[pattern]
@@ -568,7 +584,8 @@ class ResourceRegistry(object):
                   cnxt=None,
                   support_status=None,
                   type_name=None,
-                  version=None):
+                  version=None,
+                  with_description=False):
         """Return a list of valid resource types."""
 
         # validate the support status
@@ -592,7 +609,7 @@ class ResourceRegistry(object):
                 return True
 
             try:
-                return cls.get_class().is_service_available(cnxt)
+                return cls.get_class().is_service_available(cnxt)[0]
             except Exception:
                 return False
 
@@ -621,7 +638,21 @@ class ResourceRegistry(object):
             return (version is None or
                     cls.get_class().support_status.version == version)
 
-        return [name for name, cls in six.iteritems(self._registry)
+        def resource_description(name, cls, with_description):
+            if not with_description:
+                return name
+            if cls.description == 'Plugin':
+                rsrc = cls.value
+            elif cls.description == 'Template':
+                rsrc = cls.get_class()
+            else:
+                rsrc = None
+            return {
+                'resource_type': name,
+                'description': rsrc.__doc__}
+
+        return [resource_description(name, cls, with_description)
+                for name, cls in six.iteritems(self._registry)
                 if (is_resource(name) and
                     name_matches(name) and
                     status_matches(cls) and
@@ -654,11 +685,11 @@ class Environment(object):
             global_registry = None
             event_sink_classes = {}
 
-        self.registry = ResourceRegistry(global_registry, self)
-        self.registry.load(env.get(env_fmt.RESOURCE_REGISTRY, {}))
-
         self.param_defaults = env.get(env_fmt.PARAMETER_DEFAULTS, {})
 
+        self.registry = ResourceRegistry(global_registry, self.param_defaults)
+        self.registry.load(env.get(env_fmt.RESOURCE_REGISTRY, {}))
+
         self.encrypted_param_names = env.get(env_fmt.ENCRYPTED_PARAM_NAMES, [])
 
         if env_fmt.PARAMETERS in env:
@@ -683,12 +714,21 @@ class Environment(object):
             env_snippet.get(env_fmt.PARAMETER_DEFAULTS, {}))
         self._update_event_sinks(env_snippet.get(env_fmt.EVENT_SINKS, []))
 
+    def env_as_dict(self):
+        """Get the entire environment as a dict."""
+        user_env = self.user_env_as_dict()
+        user_env.update(
+            # Any data here is to be stored in the DB but not reflected
+            # as part of the user environment (e.g to pass to nested stacks
+            # or made visible to the user via API calls etc
+            {env_fmt.ENCRYPTED_PARAM_NAMES: self.encrypted_param_names})
+        return user_env
+
     def user_env_as_dict(self):
-        """Get the environment as a dict, ready for storing in the db."""
+        """Get the environment as a dict, only user-allowed keys."""
         return {env_fmt.RESOURCE_REGISTRY: self.registry.as_dict(),
                 env_fmt.PARAMETERS: self.params,
                 env_fmt.PARAMETER_DEFAULTS: self.param_defaults,
-                env_fmt.ENCRYPTED_PARAM_NAMES: self.encrypted_param_names,
                 env_fmt.EVENT_SINKS: self._event_sinks}
 
     def register_class(self, resource_type, resource_class, path=None):
@@ -717,11 +757,13 @@ class Environment(object):
                   cnxt=None,
                   support_status=None,
                   type_name=None,
-                  version=None):
+                  version=None,
+                  with_description=False):
         return self.registry.get_types(cnxt,
                                        support_status=support_status,
                                        type_name=type_name,
-                                       version=version)
+                                       version=version,
+                                       with_description=with_description)
 
     def get_resource_info(self, resource_type, resource_name=None,
                           registry_type=None, ignore=None):
@@ -797,9 +839,8 @@ def read_global_environment(env, env_dir=None):
 
     try:
         env_files = glob.glob(os.path.join(env_dir, '*'))
-    except OSError as osex:
-        LOG.error(_LE('Failed to read %s'), env_dir)
-        LOG.exception(osex)
+    except OSError:
+        LOG.exception(_LE('Failed to read %s'), env_dir)
         return
 
     for file_path in env_files:
@@ -809,11 +850,7 @@ def read_global_environment(env, env_dir=None):
                 env_body = env_fmt.parse(env_fd.read())
                 env_fmt.default_for_missing(env_body)
                 env.load(env_body)
-        except ValueError as vex:
-            LOG.error(_LE('Failed to parse %(file_path)s'), {
-                      'file_path': file_path})
-            LOG.exception(vex)
-        except IOError as ioex:
-            LOG.error(_LE('Failed to read %(file_path)s'), {
-                      'file_path': file_path})
-            LOG.exception(ioex)
+        except ValueError:
+            LOG.exception(_LE('Failed to parse %s'), file_path)
+        except IOError:
+            LOG.exception(_LE('Failed to read %s'), file_path)
