@@ -14,24 +14,54 @@ import collections
 import copy
 import itertools
 import operator
-from oslo_log import log
+import warnings
+
 import six
 
 from heat.common import exception
-from heat.common.i18n import _LW
-from heat.common.i18n import repr_wraper
+from heat.common.i18n import repr_wrapper
 from heat.engine import function
 from heat.engine import properties
 
-LOG = log.getLogger(__name__)
 
 __all__ = ['ResourceDefinition']
 
 
-@repr_wraper
+@repr_wrapper
 class ResourceDefinitionCore(object):
     """A definition of a resource, independent of any template format."""
 
+    class Diff(object):
+        """A diff between two versions of the same resource definition."""
+
+        def __init__(self, old_defn, new_defn):
+            if not (isinstance(old_defn, ResourceDefinitionCore) and
+                    isinstance(new_defn, ResourceDefinitionCore)):
+                raise TypeError
+
+            self.old_defn = old_defn
+            self.new_defn = new_defn
+
+        def properties_changed(self):
+            """Return True if the resource properties have changed."""
+            return self.old_defn._properties != self.new_defn._properties
+
+        def metadata_changed(self):
+            """Return True if the resource metadata has changed."""
+            return self.old_defn._metadata != self.new_defn._metadata
+
+        def update_policy_changed(self):
+            """Return True if the resource update policy has changed."""
+            return self.old_defn._update_policy != self.new_defn._update_policy
+
+        def __bool__(self):
+            """Return True if anything has changed."""
+            return (self.properties_changed() or
+                    self.metadata_changed() or
+                    self.update_policy_changed())
+
+        __nonzero__ = __bool__
+
     DELETION_POLICIES = (
         DELETE, RETAIN, SNAPSHOT,
     ) = (
@@ -40,7 +70,7 @@ class ResourceDefinitionCore(object):
 
     def __init__(self, name, resource_type, properties=None, metadata=None,
                  depends=None, deletion_policy=None, update_policy=None,
-                 description=None):
+                 description=None, external_id=None, condition=None):
         """Initialise with the parsed definition of a resource.
 
         Any intrinsic functions present in any of the sections should have been
@@ -54,6 +84,8 @@ class ResourceDefinitionCore(object):
         :param deletion_policy: The deletion policy for the resource
         :param update_policy: A dictionary of supplied update policies
         :param description: A string describing the resource
+        :param external_id: A uuid of an external resource
+        :param condition: A condition name associated with the resource
         """
         self.name = name
         self.resource_type = resource_type
@@ -63,6 +95,8 @@ class ResourceDefinitionCore(object):
         self._depends = depends
         self._deletion_policy = deletion_policy
         self._update_policy = update_policy
+        self._external_id = external_id
+        self._condition = condition
 
         self._hash = hash(self.resource_type)
         self._rendering = None
@@ -94,6 +128,17 @@ class ResourceDefinitionCore(object):
                                               function.Function))
             self._hash ^= _hash_data(update_policy)
 
+        if external_id is not None:
+            assert isinstance(external_id, (six.string_types,
+                                            function.Function))
+            self._hash ^= _hash_data(external_id)
+            self._deletion_policy = self.RETAIN
+
+        if condition is not None:
+            assert isinstance(condition, (six.string_types, bool,
+                                          function.Function))
+            self._hash ^= _hash_data(condition)
+
     def freeze(self, **overrides):
         """Return a frozen resource definition, with all functions resolved.
 
@@ -101,6 +146,9 @@ class ResourceDefinitionCore(object):
         intrinsic functions). Named arguments passed to this method override
         the values passed as arguments to the constructor.
         """
+        if getattr(self, '_frozen', False) and not overrides:
+            return self
+
         def arg_item(attr_name):
             name = attr_name.lstrip('_')
             if name in overrides:
@@ -114,7 +162,7 @@ class ResourceDefinitionCore(object):
 
         args = ('name', 'resource_type', '_properties', '_metadata',
                 '_depends', '_deletion_policy', '_update_policy',
-                'description')
+                'description', '_external_id', '_condition')
 
         defn = type(self)(**dict(arg_item(a) for a in args))
         defn._frozen = True
@@ -138,7 +186,9 @@ class ResourceDefinitionCore(object):
             metadata=reparse_snippet(self._metadata),
             depends=reparse_snippet(self._depends),
             deletion_policy=reparse_snippet(self._deletion_policy),
-            update_policy=reparse_snippet(self._update_policy))
+            update_policy=reparse_snippet(self._update_policy),
+            external_id=reparse_snippet(self._external_id),
+            condition=self._condition)
 
     def dep_attrs(self, resource_name):
         """Iterate over attributes of a given resource that this references.
@@ -158,21 +208,36 @@ class ResourceDefinitionCore(object):
 
         def get_resource(res_name):
             if res_name not in stack:
+                if res_name in stack.t.get(stack.t.RESOURCES):
+                    # The resource is conditionally defined, allow dependencies
+                    # on it
+                    return
                 raise exception.InvalidTemplateReference(resource=res_name,
                                                          key=self.name)
             return stack[res_name]
 
         def strict_func_deps(data, datapath):
-            return six.moves.filter(lambda r: getattr(r, 'strict_dependency',
-                                                      True),
-                                    function.dependencies(data, datapath))
+            return six.moves.filter(
+                lambda r: getattr(r, 'strict_dependency', True),
+                function.dependencies(data, datapath))
 
         explicit_depends = [] if self._depends is None else self._depends
-        return itertools.chain((get_resource(dep) for dep in explicit_depends),
-                               strict_func_deps(self._properties,
-                                                path(PROPERTIES)),
-                               strict_func_deps(self._metadata,
-                                                path(METADATA)))
+        prop_deps = strict_func_deps(self._properties, path(PROPERTIES))
+        metadata_deps = strict_func_deps(self._metadata, path(METADATA))
+
+        # (ricolin) External resource should not depend on any other resources.
+        # This operation is not allowed for now.
+        if self.external_id():
+            if explicit_depends:
+                raise exception.InvalidExternalResourceDependency(
+                    external_id=self.external_id(),
+                    resource_type=self.resource_type
+                )
+            return itertools.chain()
+
+        return itertools.chain(
+            filter(None, (get_resource(dep) for dep in explicit_depends)),
+            prop_deps, metadata_deps)
 
     def properties(self, schema, context=None):
         """Return a Properties object representing the resource properties.
@@ -205,6 +270,17 @@ class ResourceDefinitionCore(object):
         """Return the resource metadata."""
         return function.resolve(self._metadata) or {}
 
+    def external_id(self):
+        """Return the external resource id."""
+        return function.resolve(self._external_id)
+
+    def condition(self):
+        """Return the name of the conditional inclusion rule, if any.
+
+        Returns None if the resource is included unconditionally.
+        """
+        return function.resolve(self._condition)
+
     def render_hot(self):
         """Return a HOT snippet for the resource definition."""
         if self._rendering is None:
@@ -215,6 +291,8 @@ class ResourceDefinitionCore(object):
                 'deletion_policy': '_deletion_policy',
                 'update_policy': '_update_policy',
                 'depends_on': '_depends',
+                'external_id': '_external_id',
+                'condition': '_condition'
             }
 
             def rawattrs():
@@ -228,6 +306,17 @@ class ResourceDefinitionCore(object):
 
         return self._rendering
 
+    def __sub__(self, previous):
+        """Calculate the difference between this definition and a previous one.
+
+        Return a Diff object that can be used to establish differences between
+        this definition and a previous definition of the same resource.
+        """
+        if not isinstance(previous, ResourceDefinitionCore):
+            return NotImplemented
+
+        return self.Diff(previous, self)
+
     def __eq__(self, other):
         """Compare this resource definition for equality with another.
 
@@ -268,7 +357,7 @@ class ResourceDefinitionCore(object):
             return '='.join([arg_name, repr(getattr(self, '_%s' % arg_name))])
 
         args = ('properties', 'metadata', 'depends',
-                'deletion_policy', 'update_policy')
+                'deletion_policy', 'update_policy', 'condition')
         data = {
             'classname': type(self).__name__,
             'name': repr(self.name),
@@ -291,17 +380,62 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
     """A resource definition that also acts like a cfn template snippet.
 
     This class exists only for backwards compatibility with existing resource
-    plugins and unit tests; it is deprecated and then could be replaced with
-    ResourceDefinitionCore as soon as M release.
+    plugins and unit tests; it is deprecated and will be replaced with
+    ResourceDefinitionCore, possibly as soon as the Ocata release.
     """
 
-    _deprecation_msg = _LW(
+    _deprecation_msg = (
         'Reading the ResourceDefinition as if it were a snippet of a '
         'CloudFormation template is deprecated, and the ability to treat it '
         'as such will be removed in the future. Resource plugins should use '
         'the ResourceDefinition API to work with the definition of the '
         'resource instance.')
 
+    class Diff(ResourceDefinitionCore.Diff, collections.Mapping):
+        """A resource definition diff that acts like a cfn template snippet.
+
+        This class exists only for backwards compatibility with existing
+        resource plugins and unit tests; it is deprecated and could be removed
+        as soon as the Ocata release. Prefer using the API directly rather than
+        treating the diff as a dict containing the differences between two cfn
+        template snippets.
+        """
+
+        _deprecation_msg = (
+            'Reading the ResourceDefinition Diff as if it were a diff of two '
+            'snippets from CloudFormation templates is deprecated, and the '
+            'ability to treat it as such will be removed in the future. '
+            'Resource plugins should use the ResourceDefinition.Diff API and '
+            'the ResourceDefinition API to detect changes in the definition '
+            'and work with the new definition of the resource.')
+
+        def __contains__(self, key):
+            warnings.warn(self._deprecation_msg, DeprecationWarning)
+
+            if key == PROPERTIES:
+                return self.properties_changed()
+            elif key == METADATA:
+                return self.metadata_changed()
+            elif key == UPDATE_POLICY:
+                return self.update_policy_changed()
+            else:
+                return False
+
+        def __iter__(self):
+            return (k for k in _KEYS if k in self)
+
+        def __getitem__(self, key):
+            if key not in self:
+                raise KeyError
+            return self.new_defn.get(key)
+
+        def __len__(self):
+            return len(list(iter(self)))
+
+        def __repr__(self):
+            """Return a string representation of the diff."""
+            return 'ResourceDefinition.Diff %s' % repr(dict(self))
+
     def __eq__(self, other):
         """Compare this resource definition for equality with another.
 
@@ -330,7 +464,7 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
         This is for backwards compatibility with existing code that expects a
         parsed-JSON template snippet.
         """
-        LOG.warning(self._deprecation_msg)
+        warnings.warn(self._deprecation_msg, DeprecationWarning)
 
         yield TYPE
         if self._properties is not None:
@@ -352,7 +486,7 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
         This is for backwards compatibility with existing code that expects a
         parsed-JSON template snippet.
         """
-        LOG.warning(self._deprecation_msg)
+        warnings.warn(self._deprecation_msg, DeprecationWarning)
 
         if key == TYPE:
             return self.resource_type
@@ -385,8 +519,6 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
         This is for backwards compatibility with existing code that expects a
         parsed-JSON template snippet.
         """
-        LOG.warning(self._deprecation_msg)
-
         return len(list(iter(self)))
 
     def __repr__(self):
