@@ -364,15 +364,19 @@ class Stack(collections.Mapping):
         if tid is None:
             tid = self.t.id
 
+        if tid == self.t.id:
+            cur_res = self.resources.get(db_res.name)
+            if cur_res is not None and (cur_res.id == db_res.id):
+                return cur_res
+
         if rsrc_def_cache and tid in rsrc_def_cache:
             rsrc_def = rsrc_def_cache[tid]
-        elif tid == self.t.id:
-            rsrc_def = self.t.resource_definitions(self)
-            if rsrc_def_cache:
-                rsrc_def_cache[tid] = rsrc_def
         else:
-            t = tmpl.Template.load(self.context, tid)
-            rsrc_def = t.resource_definitions(self)
+            if tid == self.t.id:
+                rsrc_def = self.t.resource_definitions(self)
+            else:
+                t = tmpl.Template.load(self.context, tid)
+                rsrc_def = t.resource_definitions(self)
             if rsrc_def_cache:
                 rsrc_def_cache[tid] = rsrc_def
 
@@ -700,8 +704,7 @@ class Stack(collections.Mapping):
         self.t.add_resource(definition)
         if self.t.id is not None:
             self.t.store(self.context)
-        if resource.action == resource.INIT:
-            resource._store()
+        resource.store()
 
     def remove_resource(self, resource_name):
         """Remove the resource with the specified name."""
@@ -982,7 +985,7 @@ class Stack(collections.Mapping):
     def _store_resources(self):
         for r in reversed(self.dependencies):
             if r.action == r.INIT:
-                r._store()
+                r.store()
 
     @profiler.trace('Stack.create', hide_args=False)
     @reset_state_on_error
@@ -1353,7 +1356,7 @@ class Stack(collections.Mapping):
             if existing_rsrc_db is None:
                 update_needed_by(rsrc)
                 rsrc.current_template_id = self.t.id
-                rsrc._store()
+                rsrc.store()
                 rsrcs[rsrc.name] = rsrc
             else:
                 update_needed_by(existing_rsrc_db)
@@ -1520,9 +1523,14 @@ class Stack(collections.Mapping):
                 # and new stack resources, we should have user params of both.
                 existing_params.load(newstack.t.env.user_env_as_dict())
                 self.t.env = existing_params
+                # Update the template version, in case new things were used
+                self.t.t[newstack.t.version[0]] = max(
+                    newstack.t.version[1], self.t.version[1])
                 self.t.merge_snippets(newstack.t)
                 self.t.store(self.context)
                 backup_stack.t.env = existing_params
+                backup_stack.t.t[newstack.t.version[0]] = max(
+                    newstack.t.version[1], self.t.version[1])
                 backup_stack.t.merge_snippets(newstack.t)
                 backup_stack.t.store(self.context)
             self.store()
