@@ -15,6 +15,8 @@
 
 """Resource object."""
 
+import collections
+
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 from oslo_versionedobjects import base
@@ -29,7 +31,6 @@ from heat.db import api as db_api
 from heat.objects import base as heat_base
 from heat.objects import fields as heat_fields
 from heat.objects import resource_data
-from heat.objects import stack
 
 cfg.CONF.import_opt('encrypt_parameters_and_properties', 'heat.common.config')
 
@@ -37,12 +38,26 @@ cfg.CONF.import_opt('encrypt_parameters_and_properties', 'heat.common.config')
 def retry_on_conflict(func):
     def is_conflict(ex):
         return isinstance(ex, exception.ConcurrentTransaction)
+
     wrapper = retrying.retry(stop_max_attempt_number=11,
-                             wait_random_min=0.0, wait_random_max=2.0,
+                             wait_random_min=0, wait_random_max=2000,
                              retry_on_exception=is_conflict)
     return wrapper(func)
 
 
+class ResourceCache(object):
+
+    def __init__(self):
+        self.delete_all()
+
+    def delete_all(self):
+        self.by_stack_id_name = collections.defaultdict(dict)
+
+    def set_by_stack_id(self, resources):
+        for res in six.itervalues(resources):
+            self.by_stack_id_name[res.stack_id][res.name] = res
+
+
 class Resource(
     heat_base.HeatObject,
     base.VersionedObjectDictCompat,
@@ -54,7 +69,7 @@ class Resource(
         'stack_id': fields.StringField(),
         'created_at': fields.DateTimeField(read_only=True),
         'updated_at': fields.DateTimeField(nullable=True),
-        'nova_instance': fields.StringField(nullable=True),
+        'physical_resource_id': fields.StringField(nullable=True),
         'name': fields.StringField(nullable=True),
         'status': fields.StringField(nullable=True),
         'status_reason': fields.StringField(nullable=True),
@@ -66,7 +81,6 @@ class Resource(
             resource_data.ResourceData,
             nullable=True
         ),
-        'stack': fields.ObjectField(stack.Stack),
         'engine_id': fields.StringField(nullable=True),
         'atomic_key': fields.IntegerField(nullable=True),
         'current_template_id': fields.IntegerField(),
@@ -103,8 +117,9 @@ class Resource(
         return resource
 
     @classmethod
-    def get_obj(cls, context, resource_id):
-        resource_db = db_api.resource_get(context, resource_id)
+    def get_obj(cls, context, resource_id, refresh=False):
+        resource_db = db_api.resource_get(context, resource_id,
+                                          refresh=refresh)
         return cls._from_db_object(cls(context), context, resource_db)
 
     @classmethod
@@ -126,8 +141,7 @@ class Resource(
 
     @classmethod
     def delete(cls, context, resource_id):
-        resource_db = db_api.resource_get(context, resource_id)
-        resource_db.delete()
+        db_api.resource_delete(context, resource_id)
 
     @classmethod
     def exchange_stacks(cls, context, resource_id1, resource_id2):
@@ -137,19 +151,54 @@ class Resource(
             resource_id2)
 
     @classmethod
-    def get_all_by_stack(cls, context, stack_id, key_id=False, filters=None):
-        resources_db = db_api.resource_get_all_by_stack(context,
-                                                        stack_id, key_id,
+    def get_all_by_stack(cls, context, stack_id, filters=None):
+        cache = context.cache(ResourceCache)
+        resources = cache.by_stack_id_name.get(stack_id)
+        if resources:
+            return dict(resources)
+        resources_db = db_api.resource_get_all_by_stack(context, stack_id,
                                                         filters)
+        return cls._resources_to_dict(context, resources_db)
+
+    @classmethod
+    def _resources_to_dict(cls, context, resources_db):
         resources = [
             (
-                resource_key,
+                resource_name,
                 cls._from_db_object(cls(context), context, resource_db)
             )
-            for resource_key, resource_db in six.iteritems(resources_db)
+            for resource_name, resource_db in six.iteritems(resources_db)
         ]
         return dict(resources)
 
+    @classmethod
+    def get_all_active_by_stack(cls, context, stack_id):
+        resources_db = db_api.resource_get_all_active_by_stack(context,
+                                                               stack_id)
+        resources = [
+            (
+                resource_id,
+                cls._from_db_object(cls(context), context, resource_db)
+            )
+            for resource_id, resource_db in six.iteritems(resources_db)
+        ]
+        return dict(resources)
+
+    @classmethod
+    def get_all_by_root_stack(cls, context, stack_id, filters, cache=False):
+        resources_db = db_api.resource_get_all_by_root_stack(
+            context,
+            stack_id,
+            filters)
+        all = cls._resources_to_dict(context, resources_db)
+        if cache:
+            context.cache(ResourceCache).set_by_stack_id(all)
+        return all
+
+    @classmethod
+    def purge_deleted(cls, context, stack_id):
+        return db_api.resource_purge_deleted(context, stack_id)
+
     @classmethod
     def get_by_name_and_stack(cls, context, resource_name, stack_id):
         resource_db = db_api.resource_get_by_name_and_stack(
@@ -167,12 +216,10 @@ class Resource(
 
     @classmethod
     def update_by_id(cls, context, resource_id, values):
-        resource_db = db_api.resource_get(context, resource_id)
-        resource_db.update_and_save(values)
+        db_api.resource_update_and_save(context, resource_id, values)
 
     def update_and_save(self, values):
-        resource_db = db_api.resource_get(self._context, self.id)
-        resource_db.update_and_save(values)
+        db_api.resource_update_and_save(self._context, self.id, values)
 
     def select_and_update(self, values, expected_engine_id=None,
                           atomic_key=0):
@@ -180,9 +227,8 @@ class Resource(
                                       atomic_key=atomic_key,
                                       expected_engine_id=expected_engine_id)
 
-    def refresh(self, attrs=None):
-        resource_db = db_api.resource_get(self._context, self.id)
-        resource_db.refresh(attrs=attrs)
+    def refresh(self):
+        resource_db = db_api.resource_get(self._context, self.id, refresh=True)
         return self.__class__._from_db_object(
             self,
             self._context,
