@@ -78,9 +78,9 @@ RES_KEYS = (
 
 RES_SCHEMA_KEYS = (
     RES_SCHEMA_RES_TYPE, RES_SCHEMA_PROPERTIES, RES_SCHEMA_ATTRIBUTES,
-    RES_SCHEMA_SUPPORT_STATUS,
+    RES_SCHEMA_SUPPORT_STATUS, RES_SCHEMA_DESCRIPTION
 ) = (
-    RES_TYPE, 'properties', 'attributes', 'support_status'
+    RES_TYPE, 'properties', 'attributes', 'support_status', 'description'
 )
 
 EVENT_KEYS = (
@@ -89,14 +89,14 @@ EVENT_KEYS = (
     EVENT_TIMESTAMP,
     EVENT_RES_NAME, EVENT_RES_PHYSICAL_ID, EVENT_RES_ACTION,
     EVENT_RES_STATUS, EVENT_RES_STATUS_DATA, EVENT_RES_TYPE,
-    EVENT_RES_PROPERTIES,
+    EVENT_RES_PROPERTIES, EVENT_ROOT_STACK_ID
 ) = (
     'event_identity',
     STACK_ID, STACK_NAME,
     'event_time',
     RES_NAME, RES_PHYSICAL_ID, RES_ACTION,
     RES_STATUS, RES_STATUS_DATA, RES_TYPE,
-    'resource_properties',
+    'resource_properties', 'root_stack_id'
 )
 
 NOTIFY_KEYS = (
@@ -109,6 +109,9 @@ NOTIFY_KEYS = (
     NOTIFY_STATE,
     NOTIFY_STATE_REASON,
     NOTIFY_CREATE_AT,
+    NOTIFY_DESCRIPTION,
+    NOTIFY_UPDATE_AT,
+    NOTIFY_TAGS,
 ) = (
     'tenant_id',
     'user_id',
@@ -119,6 +122,9 @@ NOTIFY_KEYS = (
     'state',
     'state_reason',
     'create_at',
+    STACK_DESCRIPTION,
+    'updated_at',
+    STACK_TAGS,
 )
 
 # This is the representation of a watch we expose to the API via RPC
