@@ -47,6 +47,16 @@ class EngineClient(object):
         1.24 - Adds ignorable_errors to validate_template
         1.25 - list_stack_resource filter update
         1.26 - Add mark_unhealthy
+        1.27 - Add check_software_deployment
+        1.28 - Add get_environment call
+        1.29 - Add template_id to create_stack/update_stack
+        1.30 - Add possibility to resource_type_* return descriptions
+        1.31 - Add nested_depth to list_events, when nested_depth is specified
+               add root_stack_id to response
+        1.32 - Add get_files call
+        1.33 - Remove tenant_safe from list_stacks, count_stacks
+               and list_software_configs
+        1.34 - Add migrate_convergence_1 call
     """
 
     BASE_RPC_API_VERSION = '1.0'
@@ -60,12 +70,17 @@ class EngineClient(object):
     def make_msg(method, **kwargs):
         return method, kwargs
 
-    def call(self, ctxt, msg, version=None):
+    def call(self, ctxt, msg, version=None, timeout=None):
         method, kwargs = msg
+
         if version is not None:
             client = self._client.prepare(version=version)
         else:
             client = self._client
+
+        if timeout is not None:
+            client = client.prepare(timeout=timeout)
+
         return client.call(ctxt, method, **kwargs)
 
     def cast(self, ctxt, msg, version=None):
@@ -104,7 +119,7 @@ class EngineClient(object):
                                              stack_name=stack_name))
 
     def list_stacks(self, ctxt, limit=None, marker=None, sort_keys=None,
-                    sort_dir=None, filters=None, tenant_safe=True,
+                    sort_dir=None, filters=None,
                     show_deleted=False, show_nested=False, show_hidden=False,
                     tags=None, tags_any=None, not_tags=None,
                     not_tags_any=None):
@@ -120,7 +135,6 @@ class EngineClient(object):
         :param sort_keys: an array of fields used to sort the list
         :param sort_dir: the direction of the sort ('asc' or 'desc')
         :param filters: a dict with attribute:value to filter the list
-        :param tenant_safe: if true, scope the request by the current tenant
         :param show_deleted: if true, show soft-deleted stacks
         :param show_nested: if true, show nested stacks
         :param show_hidden: if true, show hidden stacks
@@ -138,16 +152,15 @@ class EngineClient(object):
                          self.make_msg('list_stacks', limit=limit,
                                        sort_keys=sort_keys, marker=marker,
                                        sort_dir=sort_dir, filters=filters,
-                                       tenant_safe=tenant_safe,
                                        show_deleted=show_deleted,
                                        show_nested=show_nested,
                                        show_hidden=show_hidden,
                                        tags=tags, tags_any=tags_any,
                                        not_tags=not_tags,
                                        not_tags_any=not_tags_any),
-                         version='1.8')
+                         version='1.33')
 
-    def count_stacks(self, ctxt, filters=None, tenant_safe=True,
+    def count_stacks(self, ctxt, filters=None,
                      show_deleted=False, show_nested=False, show_hidden=False,
                      tags=None, tags_any=None, not_tags=None,
                      not_tags_any=None):
@@ -155,7 +168,6 @@ class EngineClient(object):
 
         :param ctxt: RPC context.
         :param filters: a dict of ATTR:VALUE to match against stacks
-        :param tenant_safe: if true, scope the request by the current tenant
         :param show_deleted: if true, count will include the deleted stacks
         :param show_nested: if true, count will include nested stacks
         :param show_hidden: if true, count will include hidden stacks
@@ -171,7 +183,6 @@ class EngineClient(object):
         """
         return self.call(ctxt, self.make_msg('count_stacks',
                                              filters=filters,
-                                             tenant_safe=tenant_safe,
                                              show_deleted=show_deleted,
                                              show_nested=show_nested,
                                              show_hidden=show_hidden,
@@ -179,7 +190,7 @@ class EngineClient(object):
                                              tags_any=tags_any,
                                              not_tags=not_tags,
                                              not_tags_any=not_tags_any),
-                         version='1.8')
+                         version='1.33')
 
     def show_stack(self, ctxt, stack_identity, resolve_outputs=True):
         """Returns detailed information about one or all stacks.
@@ -242,7 +253,8 @@ class EngineClient(object):
     def _create_stack(self, ctxt, stack_name, template, params, files,
                       args, environment_files=None,
                       owner_id=None, nested_depth=0, user_creds_id=None,
-                      stack_user_project_id=None, parent_resource_name=None):
+                      stack_user_project_id=None, parent_resource_name=None,
+                      template_id=None):
         """Internal interface for engine-to-engine communication via RPC.
 
         Allows some additional options which should not be exposed to users via
@@ -253,6 +265,7 @@ class EngineClient(object):
         :param user_creds_id: user_creds record for nested stack
         :param stack_user_project_id: stack user project for nested stack
         :param parent_resource_name: the parent resource name
+        :param template_id: the ID of a pre-stored template in the DB
         """
         return self.call(
             ctxt, self.make_msg('create_stack', stack_name=stack_name,
@@ -263,8 +276,9 @@ class EngineClient(object):
                                 nested_depth=nested_depth,
                                 user_creds_id=user_creds_id,
                                 stack_user_project_id=stack_user_project_id,
-                                parent_resource_name=parent_resource_name),
-            version='1.23')
+                                parent_resource_name=parent_resource_name,
+                                template_id=template_id),
+            version='1.29')
 
     def update_stack(self, ctxt, stack_identity, template, params,
                      files, args, environment_files=None):
@@ -283,6 +297,20 @@ class EngineClient(object):
                names included in the files dict
         :type  environment_files: list or None
         """
+        return self._update_stack(ctxt, stack_identity, template, params,
+                                  files, args,
+                                  environment_files=environment_files)
+
+    def _update_stack(self, ctxt, stack_identity, template, params,
+                      files, args, environment_files=None,
+                      template_id=None):
+        """Internal interface for engine-to-engine communication via RPC.
+
+        Allows an additional option which should not be exposed to users via
+        the API:
+
+        :param template_id: the ID of a pre-stored template in the DB
+        """
         return self.call(ctxt,
                          self.make_msg('update_stack',
                                        stack_identity=stack_identity,
@@ -290,8 +318,9 @@ class EngineClient(object):
                                        params=params,
                                        files=files,
                                        environment_files=environment_files,
-                                       args=args),
-                         version='1.23')
+                                       args=args,
+                                       template_id=template_id),
+                         version='1.29')
 
     def preview_update_stack(self, ctxt, stack_identity, template, params,
                              files, args, environment_files=None):
@@ -366,6 +395,32 @@ class EngineClient(object):
         return self.call(ctxt, self.make_msg('get_template',
                                              stack_identity=stack_identity))
 
+    def get_environment(self, context, stack_identity):
+        """Returns the environment for an existing stack.
+
+        :param context: RPC context
+        :param stack_identity: identifies the stack
+        :rtype: dict
+        """
+
+        return self.call(context,
+                         self.make_msg('get_environment',
+                                       stack_identity=stack_identity),
+                         version='1.28')
+
+    def get_files(self, context, stack_identity):
+        """Returns the files for an existing stack.
+
+        :param context: RPC context
+        :param stack_identity: identifies the stack
+        :rtype: dict
+        """
+
+        return self.call(context,
+                         self.make_msg('get_files',
+                                       stack_identity=stack_identity),
+                         version='1.32')
+
     def delete_stack(self, ctxt, stack_identity, cast=True):
         """Deletes a given stack.
 
@@ -392,19 +447,23 @@ class EngineClient(object):
                             ctxt,
                             support_status=None,
                             type_name=None,
-                            heat_version=None):
+                            heat_version=None,
+                            with_description=False):
         """Get a list of valid resource types.
 
         :param ctxt: RPC context.
         :param support_status: Support status of resource type
         :param type_name: Resource type's name (regular expression allowed)
-        :param version: Heat version
+        :param heat_version: Heat version
+        :param with_description: Either return resource type description or not
         """
-        return self.call(ctxt, self.make_msg('list_resource_types',
-                                             support_status=support_status,
-                                             type_name=type_name,
-                                             heat_version=heat_version),
-                         version='1.16')
+        return self.call(ctxt,
+                         self.make_msg('list_resource_types',
+                                       support_status=support_status,
+                                       type_name=type_name,
+                                       heat_version=heat_version,
+                                       with_description=with_description),
+                         version='1.30')
 
     def list_template_versions(self, ctxt):
         """Get a list of available template versions.
@@ -425,13 +484,17 @@ class EngineClient(object):
             'list_template_functions', template_version=template_version),
             version='1.13')
 
-    def resource_schema(self, ctxt, type_name):
+    def resource_schema(self, ctxt, type_name, with_description=False):
         """Get the schema for a resource type.
 
         :param ctxt: RPC context.
+        :param with_description: Return resource with description or not.
         """
-        return self.call(ctxt, self.make_msg('resource_schema',
-                                             type_name=type_name))
+        return self.call(ctxt,
+                         self.make_msg('resource_schema',
+                                       type_name=type_name,
+                                       with_description=with_description),
+                         version='1.30')
 
     def generate_template(self, ctxt, type_name, template_type='cfn'):
         """Generate a template based on the specified type.
@@ -446,7 +509,8 @@ class EngineClient(object):
                          version='1.9')
 
     def list_events(self, ctxt, stack_identity, filters=None, limit=None,
-                    marker=None, sort_keys=None, sort_dir=None,):
+                    marker=None, sort_keys=None, sort_dir=None,
+                    nested_depth=None):
         """Lists all events associated with a given stack.
 
         It supports pagination (``limit`` and ``marker``),
@@ -460,6 +524,7 @@ class EngineClient(object):
         :param marker: the ID of the last event in the previous page
         :param sort_keys: an array of fields used to sort the list
         :param sort_dir: the direction of the sort ('asc' or 'desc').
+        :param nested_depth: Levels of nested stacks to list events for.
         """
         return self.call(ctxt, self.make_msg('list_events',
                                              stack_identity=stack_identity,
@@ -467,7 +532,9 @@ class EngineClient(object):
                                              limit=limit,
                                              marker=marker,
                                              sort_keys=sort_keys,
-                                             sort_dir=sort_dir))
+                                             sort_dir=sort_dir,
+                                             nested_depth=nested_depth),
+                         version='1.31')
 
     def describe_stack_resource(self, ctxt, stack_identity, resource_name,
                                 with_attr=False):
@@ -643,14 +710,12 @@ class EngineClient(object):
         return self.call(cnxt, self.make_msg('show_software_config',
                                              config_id=config_id))
 
-    def list_software_configs(self, cnxt, limit=None, marker=None,
-                              tenant_safe=True):
+    def list_software_configs(self, cnxt, limit=None, marker=None):
         return self.call(cnxt,
                          self.make_msg('list_software_configs',
                                        limit=limit,
-                                       marker=marker,
-                                       tenant_safe=tenant_safe),
-                         version='1.10')
+                                       marker=marker),
+                         version='1.33')
 
     def create_software_config(self, cnxt, group, name, config,
                                inputs=None, outputs=None, options=None):
@@ -681,6 +746,12 @@ class EngineClient(object):
         return self.call(cnxt, self.make_msg('show_software_deployment',
                                              deployment_id=deployment_id))
 
+    def check_software_deployment(self, cnxt, deployment_id, timeout):
+        return self.call(cnxt, self.make_msg('check_software_deployment',
+                                             deployment_id=deployment_id,
+                                             timeout=timeout),
+                         timeout=timeout, version='1.27')
+
     def create_software_deployment(self, cnxt, server_id, config_id=None,
                                    input_values=None, action='INIT',
                                    status='COMPLETE', status_reason='',
@@ -776,3 +847,14 @@ class EngineClient(object):
                          self.make_msg('export_stack',
                                        stack_identity=stack_identity),
                          version='1.22')
+
+    def migrate_convergence_1(self, ctxt, stack_id):
+        """Migrate the stack to convergence engine
+
+        :param ctxt: RPC context
+        :param stack_name: Name of the stack you want to migrate
+        """
+        return self.call(ctxt,
+                         self.make_msg('migrate_convergence_1',
+                                       stack_id=stack_id),
+                         version='1.34')
