@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from heat.common import exception
 from oslo_utils import timeutils
 import six
 
@@ -55,17 +56,19 @@ class CooldownMixin(object):
         self.metadata_set(metadata)
         return True
 
-    def _finished_scaling(self, cooldown_reason,
-                          changed_size=True):
+    def _finished_scaling(self, cooldown_reason, size_changed=True):
         # If we wanted to implement the AutoScaling API like AWS does,
         # we could maintain event history here, but since we only need
         # the latest event for cooldown, just store that for now
         metadata = self.metadata_get()
-        if changed_size:
+        if size_changed:
             now = timeutils.utcnow().isoformat()
             metadata['cooldown'] = {now: cooldown_reason}
         metadata['scaling_in_progress'] = False
-        self.metadata_set(metadata)
+        try:
+            self.metadata_set(metadata)
+        except exception.NotFound:
+            pass
 
     def handle_metadata_reset(self):
         metadata = self.metadata_get()
