@@ -27,10 +27,10 @@ class SwiftClientPluginTestCase(common.HeatTestCase):
         super(SwiftClientPluginTestCase, self).setUp()
         self.swift_client = mock.Mock()
         self.context = utils.dummy_context()
-        self.context.tenant_id = "demo"
+        self.context.tenant = "demo"
         c = self.context.clients
         self.swift_plugin = c.client_plugin('swift')
-        self.swift_plugin._client = self.swift_client
+        self.swift_plugin.client = lambda: self.swift_client
 
 
 class SwiftUtilsTest(SwiftClientPluginTestCase):
