@@ -16,6 +16,8 @@ import copy
 import json
 
 from cinderclient import exceptions as cinder_exp
+import mock
+from oslo_config import cfg
 import six
 
 from heat.common import exception
@@ -100,7 +102,9 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
             size=size, availability_zone='nova',
             description='test_description',
             name='test_name',
-            metadata={'key': 'value'}).AndReturn(fv)
+            metadata={'key': 'value'},
+            multiattach=False
+        ).AndReturn(fv)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
         fv_ready = vt_base.FakeVolume(final_status, id=fv.id)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv_ready)
@@ -130,7 +134,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
             description='test_description',
             name='test_name',
             metadata={'key': 'value'},
-            volume_type='lvm').AndReturn(fv)
+            volume_type='lvm',
+            multiattach=False).AndReturn(fv)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
         fv_ready = vt_base.FakeVolume('available', id=fv.id)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv_ready)
@@ -160,7 +165,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
             size=1, availability_zone='nova',
             description='ImageVolumeDescription',
             name='ImageVolume',
-            imageRef=image_id).AndReturn(fv)
+            imageRef=image_id,
+            multiattach=False).AndReturn(fv)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
         fv_ready = vt_base.FakeVolume('available', id=fv.id)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv_ready)
@@ -188,7 +194,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=1, availability_zone='nova',
             description='ImageVolumeDescription',
-            name='ImageVolume').AndReturn(fv)
+            name='ImageVolume',
+            multiattach=False).AndReturn(fv)
 
         update_readonly_mock = self.patchobject(self.cinder_fc.volumes,
                                                 'update_readonly_flag')
@@ -222,7 +229,9 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=1, availability_zone='nova',
             description=None,
-            name=vol_name).AndReturn(fv)
+            name=vol_name,
+            multiattach=False
+        ).AndReturn(fv)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
         fv_ready = vt_base.FakeVolume('available', id=fv.id)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv_ready)
@@ -273,7 +282,6 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.assertEqual(u'False', rsrc.FnGetAtt('bootable'))
         self.assertEqual(u'False', rsrc.FnGetAtt('encrypted'))
         self.assertEqual(u'[]', rsrc.FnGetAtt('attachments'))
-        self.assertEqual({'volume': 'info'}, rsrc.FnGetAtt('show'))
         self.assertEqual('False', rsrc.FnGetAtt('multiattach'))
         error = self.assertRaises(exception.InvalidTemplateAttribute,
                                   rsrc.FnGetAtt, 'unknown')
@@ -544,21 +552,6 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.assertEqual((rsrc.UPDATE, rsrc.COMPLETE), rsrc.state)
         self.assertEqual(1, self.cinder_fc.volumes.retype.call_count)
 
-        self.cinder_fc.volume_api_version = 1
-        new_vol_type_1 = 'new_type_1'
-        props = copy.deepcopy(rsrc.properties.data)
-        props['volume_type'] = new_vol_type_1
-        after = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(), props)
-        # if the volume api is v1, not support to retype
-        update_task = scheduler.TaskRunner(rsrc.update, after)
-        ex = self.assertRaises(exception.ResourceFailure, update_task)
-        self.assertEqual('NotSupported: resources.volume2: '
-                         'Using Cinder API V1, '
-                         'volume_type update is not supported.',
-                         six.text_type(ex))
-        self.assertEqual((rsrc.UPDATE, rsrc.FAILED), rsrc.state)
-        self.assertEqual(1, self.cinder_fc.volumes.retype.call_count)
-
     def test_cinder_volume_update_name_and_metadata(self):
         # update the name, description and metadata
         fv = vt_base.FakeVolume('creating',
@@ -603,6 +596,7 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
             size=1, availability_zone='nova',
             description='test_description',
             name='test_name',
+            multiattach=False,
             metadata={u'key': u'value'}).AndReturn(fv)
 
         update_readonly_mock = self.patchobject(self.cinder_fc.volumes,
@@ -709,14 +703,15 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=1, availability_zone=None,
             description='test_description',
-            name='test_name'
+            name='test_name',
+            multiattach=False
         ).AndReturn(vt_base.FakeVolume('creating'))
         fv = vt_base.FakeVolume('available')
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
 
         fb = vt_base.FakeBackup('creating')
         self.m.StubOutWithMock(self.cinder_fc.backups, 'create')
-        self.cinder_fc.backups.create(fv.id).AndReturn(fb)
+        self.cinder_fc.backups.create(fv.id, force=True).AndReturn(fb)
         self.m.StubOutWithMock(self.cinder_fc.backups, 'get')
         self.cinder_fc.backups.get(fb.id).AndReturn(
             vt_base.FakeBackup('available'))
@@ -727,6 +722,7 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         stack = utils.parse_stack(t, stack_name=stack_name)
 
         rsrc = stack['volume']
+        self.patchobject(rsrc, '_store_config_default_properties')
         scheduler.TaskRunner(rsrc.create)()
 
         scheduler.TaskRunner(rsrc.snapshot)()
@@ -746,14 +742,15 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=1, availability_zone=None,
             description='test_description',
-            name='test_name'
+            name='test_name',
+            multiattach=False
         ).AndReturn(vt_base.FakeVolume('creating'))
         fv = vt_base.FakeVolume('available')
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
 
         fb = vt_base.FakeBackup('creating')
         self.m.StubOutWithMock(self.cinder_fc.backups, 'create')
-        self.cinder_fc.backups.create(fv.id).AndReturn(fb)
+        self.cinder_fc.backups.create(fv.id, force=True).AndReturn(fb)
         self.m.StubOutWithMock(self.cinder_fc.backups, 'get')
         fail_reason = 'Could not determine which Swift endpoint to use'
         self.cinder_fc.backups.get(fb.id).AndReturn(
@@ -765,6 +762,7 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         stack = utils.parse_stack(t, stack_name=stack_name)
 
         rsrc = stack['volume']
+        self.patchobject(rsrc, '_store_config_default_properties')
         scheduler.TaskRunner(rsrc.create)()
 
         self.assertRaises(exception.ResourceFailure,
@@ -833,7 +831,9 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=2, availability_zone='nova',
             description=None,
-            name=vol2_name).AndReturn(fv2)
+            name=vol2_name,
+            multiattach=False
+        ).AndReturn(fv2)
         self.cinder_fc.volumes.get(fv2.id).AndReturn(fv2)
         fv2 = vt_base.FakeVolume('available', id=fv2.id)
         self.cinder_fc.volumes.get(fv2.id).AndReturn(fv2)
@@ -948,7 +948,9 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=1, name='test_name', description=None,
             availability_zone='nova',
-            scheduler_hints={'hint1': 'good_advice'}).AndReturn(fv)
+            scheduler_hints={'hint1': 'good_advice'},
+            multiattach=False
+        ).AndReturn(fv)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
         fv_ready = vt_base.FakeVolume('available', id=fv.id)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv_ready)
@@ -957,6 +959,7 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
 
         stack_name = 'test_cvolume_scheduler_hints_stack'
         stack = utils.parse_stack(self.t, stack_name=stack_name)
+        self.patchobject(stack['volume3'], '_store_config_default_properties')
         self.create_volume(self.t, stack, 'volume3')
 
         self.m.VerifyAll()
@@ -981,23 +984,10 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
 
         self.m.VerifyAll()
 
-    def test_cinder_create_with_scheduler_hints_and_cinder_api_v1(self):
-        cinder.CinderClientPlugin._create().AndReturn(self.cinder_fc)
-        self.cinder_fc.volume_api_version = 1
-
-        self.m.ReplayAll()
-
-        stack_name = 'test_cvolume_scheduler_hints_api_v1_stack'
-        stack = utils.parse_stack(self.t, stack_name=stack_name)
-        ex = self.assertRaises(exception.StackValidationFailed,
-                               self.create_volume, self.t, stack, 'volume3')
-        self.assertIn('Scheduler hints are not supported by the current '
-                      'volume API.', six.text_type(ex))
-        self.m.VerifyAll()
-
     def test_cinder_create_with_stack_scheduler_hints(self):
         fv = vt_base.FakeVolume('creating')
-        sh.cfg.CONF.set_override('stack_scheduler_hints', True)
+        sh.cfg.CONF.set_override('stack_scheduler_hints', True,
+                                 enforce_type=True)
 
         stack_name = 'test_cvolume_stack_scheduler_hints_stack'
         t = template_format.parse(single_cinder_volume_template)
@@ -1014,10 +1004,11 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=1, name='test_name', description='test_description',
             availability_zone=None,
+            multiattach=False,
             scheduler_hints={shm.HEAT_ROOT_STACK_ID: stack.root_stack_id(),
                              shm.HEAT_STACK_ID: stack.id,
                              shm.HEAT_STACK_NAME: stack.name,
-                             shm.HEAT_PATH_IN_STACK: [(None, stack.name)],
+                             shm.HEAT_PATH_IN_STACK: [stack.name],
                              shm.HEAT_RESOURCE_NAME: rsrc.name,
                              shm.HEAT_RESOURCE_UUID: rsrc.uuid}).AndReturn(fv)
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
@@ -1025,28 +1016,13 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv_ready)
 
         self.m.ReplayAll()
+        self.patchobject(rsrc, '_store_config_default_properties')
         scheduler.TaskRunner(rsrc.create)()
         # this makes sure the auto increment worked on volume creation
         self.assertTrue(rsrc.id > 0)
 
         self.m.VerifyAll()
 
-    def test_cinder_create_with_multiattach_and_cinder_api_v1(self):
-        cinder.CinderClientPlugin._create().AndReturn(self.cinder_fc)
-        self.cinder_fc.volume_api_version = 1
-
-        self.m.ReplayAll()
-
-        stack_name = 'test_cvolume_multiattach_api_v1_stack'
-        stack = utils.parse_stack(self.t, stack_name=stack_name)
-        ex = self.assertRaises(exception.StackValidationFailed,
-                               self.create_volume, self.t, stack, 'volume4')
-        self.assertIn('Multiple attach is not supported by the current '
-                      'volume API. Use this property since '
-                      'Cinder API v2.', six.text_type(ex))
-
-        self.m.VerifyAll()
-
     def _test_cinder_create_invalid_property_combinations(
             self, stack_name, combinations, err_msg, exc):
         stack = utils.parse_stack(self.t, stack_name=stack_name)
@@ -1060,15 +1036,29 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
     def test_cinder_create_with_image_and_imageRef(self):
         stack_name = 'test_create_with_image_and_imageRef'
         combinations = {'imageRef': 'image-456', 'image': 'image-123'}
-        err_msg = "Cannot use image and imageRef at the same time."
+        err_msg = ("Cannot define the following properties at the same time: "
+                   "['image', 'imageRef'].")
         self.stub_ImageConstraint_validate()
         stack = utils.parse_stack(self.t, stack_name=stack_name)
         vp = stack.t['Resources']['volume2']['Properties']
         vp.pop('size')
         vp.update(combinations)
-        ex = self.assertRaises(ValueError, stack.get, 'volume2')
+        ex = self.assertRaises(exception.ResourcePropertyConflict,
+                               stack.get, 'volume2')
         self.assertEqual(err_msg, six.text_type(ex))
 
+    def test_cinder_create_with_image_and_size(self):
+        stack_name = 'test_create_with_image_and_size'
+        combinations = {'image': 'image-123'}
+        err_msg = ('If neither "backup_id" nor "size" is provided, one and '
+                   'only one of "source_volid", "snapshot_id" must be '
+                   'specified, but currently '
+                   'specified options: [\'image\'].')
+        self.stub_ImageConstraint_validate()
+        self._test_cinder_create_invalid_property_combinations(
+            stack_name, combinations,
+            err_msg, exception.StackValidationFailed)
+
     def test_cinder_create_with_size_snapshot_and_image(self):
         stack_name = 'test_create_with_size_snapshot_and_image'
         combinations = {
@@ -1124,8 +1114,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
             'source_volid': 'source_volume-123',
             'snapshot_id': 'snapshot-123'}
         err_msg = ('If neither "backup_id" nor "size" is provided, one and '
-                   'only one of "image", "imageRef", "source_volid", '
-                   '"snapshot_id" must be specified, but currently '
+                   'only one of "source_volid", "snapshot_id" must be '
+                   'specified, but currently '
                    'specified options: [\'snapshot_id\', \'source_volid\'].')
         self.stub_VolumeConstraint_validate()
         self.stub_SnapshotConstraint_validate()
@@ -1139,8 +1129,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
             'source_volid': 'source_volume-123',
             'image': 'image-123'}
         err_msg = ('If neither "backup_id" nor "size" is provided, one and '
-                   'only one of "image", "imageRef", "source_volid", '
-                   '"snapshot_id" must be specified, but currently '
+                   'only one of "source_volid", "snapshot_id" must be '
+                   'specified, but currently '
                    'specified options: [\'source_volid\', \'image\'].')
         self.stub_VolumeConstraint_validate()
         self.stub_ImageConstraint_validate()
@@ -1152,9 +1142,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         stack_name = 'test_create_no_size_no_options'
         combinations = {}
         err_msg = ('If neither "backup_id" nor "size" is provided, one and '
-                   'only one of "image", "imageRef", "source_volid", '
-                   '"snapshot_id" must be specified, but currently '
-                   'specified options: [].')
+                   'only one of "source_volid", "snapshot_id" must be '
+                   'specified, but currently specified options: [].')
         self._test_cinder_create_invalid_property_combinations(
             stack_name, combinations,
             err_msg, exception.StackValidationFailed)
@@ -1167,7 +1156,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.cinder_fc.volumes.create(
             size=1, availability_zone=None,
             description='test_description',
-            name='test_name'
+            name='test_name',
+            multiattach=False
         ).AndReturn(vt_base.FakeVolume('creating'))
         fv = vt_base.FakeVolume('available')
         self.cinder_fc.volumes.get(fv.id).AndReturn(fv)
@@ -1176,7 +1166,7 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         # snapshot script
         fb = vt_base.FakeBackup('creating')
         self.m.StubOutWithMock(self.cinder_fc.backups, 'create')
-        self.cinder_fc.backups.create(fv.id).AndReturn(fb)
+        self.cinder_fc.backups.create(fv.id, force=True).AndReturn(fb)
         self.m.StubOutWithMock(self.cinder_fc.backups, 'get')
         self.cinder_fc.backups.get(fb.id).AndReturn(
             vt_base.FakeBackup('available'))
@@ -1186,7 +1176,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.m.StubOutWithMock(self.cinder_fc.restores, 'restore')
         self.cinder_fc.restores.restore('backup-123',
                                         'vol-123').AndReturn(fvbr)
-        fv_restoring = vt_base.FakeVolume('restoring-backup', id=fv.id)
+        fv_restoring = vt_base.FakeVolume(
+            'restoring-backup', id=fv.id, attachments=[])
         self.cinder_fc.volumes.get('vol-123').AndReturn(fv_restoring)
         fv_final = vt_base.FakeVolume(final_status, id=fv.id)
         self.cinder_fc.volumes.get('vol-123').AndReturn(fv_final)
@@ -1195,6 +1186,8 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
 
         t = template_format.parse(single_cinder_volume_template)
         stack = utils.parse_stack(t, stack_name=stack_name)
+        self.patchobject(stack['volume'], '_store_config_default_properties')
+
         scheduler.TaskRunner(stack.create)()
 
         self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
@@ -1233,3 +1226,69 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
             stack.t.resource_definitions(stack)['volume'],
             stack)
         self.assertIsNone(rsrc.handle_delete_snapshot(mock_vs))
+
+    def test_vaildate_deletion_policy(self):
+        cfg.CONF.set_override('backups_enabled', False, group='volumes')
+        stack_name = 'test_volume_validate_deletion_policy'
+        self.t['resources']['volume']['deletion_policy'] = 'Snapshot'
+        stack = utils.parse_stack(self.t, stack_name=stack_name)
+        rsrc = self.get_volume(self.t, stack, 'volume')
+        self.assertRaisesRegex(
+            exception.StackValidationFailed,
+            'volume backup service is not enabled',
+            rsrc.validate)
+
+    def test_volume_get_live_state(self):
+        tmpl = """
+        heat_template_version: 2013-05-23
+        description: Cinder volume
+        resources:
+          volume:
+            type: OS::Cinder::Volume
+            properties:
+              size: 1
+              name: test_name
+              description: test_description
+              image: 1234
+              scheduler_hints:
+                'consistencygroup_id': 4444
+        """
+        t = template_format.parse(tmpl)
+        stack = utils.parse_stack(t, stack_name='get_live_state')
+        rsrc = stack['volume']
+        rsrc._availability_zone = 'nova'
+        rsrc.resource_id = '1234'
+
+        vol_resp = {
+            'attachments': [],
+            'availability_zone': 'nova',
+            'snapshot_id': None,
+            'size': 1,
+            'metadata': {'test': 'test_value', 'read_only': False},
+            'consistencygroup_id': '4444',
+            'volume_image_metadata': {'image_id': '1234',
+                                      'image_name': 'test'},
+            'description': None,
+            'multiattach': False,
+            'source_volid': None,
+            'name': 'test-volume-jbdbgdsy3vyg',
+            'volume_type': 'lvmdriver-1'
+        }
+        vol = mock.MagicMock()
+        vol.to_dict.return_value = vol_resp
+        rsrc.client().volumes = mock.MagicMock()
+        rsrc.client().volumes.get = mock.MagicMock(return_value=vol)
+        rsrc.client().volume_api_version = 2
+        rsrc.data = mock.MagicMock(return_value={'volume_type': 'lvmdriver-1'})
+
+        reality = rsrc.get_live_state(rsrc.properties)
+        expected = {
+            'size': 1,
+            'metadata': {'test': 'test_value'},
+            'description': None,
+            'name': 'test-volume-jbdbgdsy3vyg',
+            'backup_id': None,
+            'read_only': False,
+        }
+
+        self.assertEqual(expected, reality)
