@@ -20,13 +20,14 @@ from oslo_utils import timeutils
 import six
 
 from heat.common import exception
-from heat.common import identifier
 from heat.common import template_format
+from heat.common import timeutils as heat_timeutils
 from heat.engine import api
 from heat.engine import event
 from heat.engine import parameters
 from heat.engine import stack as parser
 from heat.engine import template
+from heat.objects import event as event_object
 from heat.rpc import api as rpc_api
 from heat.tests import common
 from heat.tests import utils
@@ -49,17 +50,19 @@ class FormatTest(common.HeatTestCase):
                 'generic4': {'Type': 'StackResourceType'}
             }
         })
-        self.stack = parser.Stack(utils.dummy_context(), 'test_stack',
+        self.context = utils.dummy_context()
+        self.stack = parser.Stack(self.context, 'test_stack',
                                   tmpl, stack_id=str(uuid.uuid4()))
 
-    def _dummy_event(self, event_id):
+    def _dummy_event(self):
         resource = self.stack['generic1']
-        return event.Event(utils.dummy_context(), self.stack, 'CREATE',
-                           'COMPLETE', 'state changed',
-                           'z3455xyc-9f88-404d-a85b-5315293e67de',
-                           resource.properties, resource.name, resource.type(),
-                           uuid='abc123yc-9f88-404d-a85b-531529456xyz',
-                           id=event_id)
+        ev = event.Event(self.context, self.stack, 'CREATE',
+                         'COMPLETE', 'state changed',
+                         'z3455xyc-9f88-404d-a85b-5315293e67de',
+                         resource.properties, resource.name, resource.type(),
+                         uuid='abc123yc-9f88-404d-a85b-531529456xyz')
+        event_id = ev.store()
+        return event_object.Event.get_by_id(self.context, event_id)
 
     def test_format_stack_resource(self):
         self.stack.created_time = datetime(2015, 8, 3, 17, 5, 1)
@@ -88,13 +91,13 @@ class FormatTest(common.HeatTestCase):
         )))
 
         formatted = api.format_stack_resource(res, True)
-        self.assertEqual(resource_details_keys, set(six.iterkeys(formatted)))
+        self.assertEqual(resource_details_keys, set(formatted.keys()))
 
         formatted = api.format_stack_resource(res, False)
-        self.assertEqual(resource_keys, set(six.iterkeys(formatted)))
-        self.assertEqual(self.stack.created_time.isoformat(),
+        self.assertEqual(resource_keys, set(formatted.keys()))
+        self.assertEqual(heat_timeutils.isotime(self.stack.created_time),
                          formatted[rpc_api.RES_CREATION_TIME])
-        self.assertEqual(self.stack.updated_time.isoformat(),
+        self.assertEqual(heat_timeutils.isotime(self.stack.updated_time),
                          formatted[rpc_api.RES_UPDATED_TIME])
         self.assertEqual(res.INIT, formatted[rpc_api.RES_ACTION])
 
@@ -206,16 +209,16 @@ class FormatTest(common.HeatTestCase):
     def test_format_stack_resource_with_nested_stack(self):
         res = self.stack['generic4']
         nested_id = {'foo': 'bar'}
-        res.nested = mock.Mock()
-        res.nested.return_value.identifier.return_value = nested_id
+        res.has_nested = mock.Mock()
+        res.has_nested.return_value = True
+        res.nested_identifier = mock.Mock()
+        res.nested_identifier.return_value = nested_id
 
         formatted = api.format_stack_resource(res, False)
         self.assertEqual(nested_id, formatted[rpc_api.RES_NESTED_STACK_ID])
 
     def test_format_stack_resource_with_nested_stack_none(self):
         res = self.stack['generic4']
-        res.nested = mock.Mock()
-        res.nested.return_value = None
 
         resource_keys = set((
             rpc_api.RES_CREATION_TIME,
@@ -232,7 +235,7 @@ class FormatTest(common.HeatTestCase):
             rpc_api.RES_REQUIRED_BY))
 
         formatted = api.format_stack_resource(res, False)
-        self.assertEqual(resource_keys, set(six.iterkeys(formatted)))
+        self.assertEqual(resource_keys, set(formatted.keys()))
 
     def test_format_stack_resource_with_nested_stack_not_found(self):
         res = self.stack['generic4']
@@ -255,18 +258,18 @@ class FormatTest(common.HeatTestCase):
 
         formatted = api.format_stack_resource(res, False)
         # 'nested_stack_id' is not in formatted
-        self.assertEqual(resource_keys, set(six.iterkeys(formatted)))
+        self.assertEqual(resource_keys, set(formatted.keys()))
 
     def test_format_stack_resource_with_nested_stack_empty(self):
         res = self.stack['generic4']
         nested_id = {'foo': 'bar'}
 
-        res.nested = mock.MagicMock()
-        res.nested.return_value.identifier.return_value = nested_id
-        res.nested.return_value.__len__.return_value = 0
+        res.has_nested = mock.Mock()
+        res.has_nested.return_value = True
+        res.nested_identifier = mock.Mock()
+        res.nested_identifier.return_value = nested_id
 
         formatted = api.format_stack_resource(res, False)
-        res.nested.return_value.identifier.assert_called_once_with()
         self.assertEqual(nested_id, formatted[rpc_api.RES_NESTED_STACK_ID])
 
     def test_format_stack_resource_required_by(self):
@@ -283,10 +286,7 @@ class FormatTest(common.HeatTestCase):
         self.assertEqual('foobar', formatted[rpc_api.RES_PARENT_RESOURCE])
 
     def test_format_event_identifier_uuid(self):
-        self._test_format_event('abc123yc-9f88-404d-a85b-531529456xyz')
-
-    def _test_format_event(self, event_id):
-        event = self._dummy_event(event_id)
+        event = self._dummy_event()
 
         event_keys = set((
             rpc_api.EVENT_ID,
@@ -301,16 +301,16 @@ class FormatTest(common.HeatTestCase):
             rpc_api.EVENT_RES_TYPE,
             rpc_api.EVENT_RES_PROPERTIES))
 
-        formatted = api.format_event(event)
-        self.assertEqual(event_keys, set(six.iterkeys(formatted)))
+        formatted = api.format_event(event, self.stack.identifier())
+        self.assertEqual(event_keys, set(formatted.keys()))
 
         event_id_formatted = formatted[rpc_api.EVENT_ID]
-        event_identifier = identifier.EventIdentifier(
-            event_id_formatted['tenant'],
-            event_id_formatted['stack_name'],
-            event_id_formatted['stack_id'],
-            event_id_formatted['path'])
-        self.assertEqual(event_id, event_identifier.event_id)
+        self.assertEqual({
+            'path': '/resources/generic1/events/%s' % event.uuid,
+            'stack_id': self.stack.id,
+            'stack_name': 'test_stack',
+            'tenant': 'test_tenant_id'
+        }, event_id_formatted)
 
     @mock.patch.object(api, 'format_stack_resource')
     def test_format_stack_preview(self, mock_fmt_resource):
@@ -348,7 +348,8 @@ class FormatTest(common.HeatTestCase):
                   'stacks/test_stack/' + self.stack.id)
         expected_stack_info = {
             'capabilities': [],
-            'creation_time': '1970-01-01T00:00:00',
+            'creation_time': '1970-01-01T00:00:00Z',
+            'deletion_time': None,
             'description': 'No description',
             'disable_rollback': True,
             'notification_topics': [],
@@ -387,7 +388,7 @@ class FormatTest(common.HeatTestCase):
 
         self.stack.updated_time = datetime(1970, 1, 1)
         info = api.format_stack(self.stack)
-        self.assertEqual('1970-01-01T00:00:00', info['updated_time'])
+        self.assertEqual('1970-01-01T00:00:00Z', info['updated_time'])
 
     @mock.patch.object(api, 'format_stack_outputs')
     def test_format_stack_adds_outputs(self, mock_fmt_outputs):
@@ -427,8 +428,7 @@ class FormatTest(common.HeatTestCase):
         stack.status = 'COMPLETE'
         stack['generic'].action = 'CREATE'
         stack['generic'].status = 'COMPLETE'
-        info = api.format_stack_outputs(stack, stack.outputs,
-                                        resolve_value=True)
+        info = api.format_stack_outputs(stack.outputs, resolve_value=True)
         expected = [{'description': 'No description given',
                      'output_error': 'The Referenced Attribute (generic Bar) '
                                      'is incorrect.',
@@ -463,7 +463,7 @@ class FormatTest(common.HeatTestCase):
         stack.status = 'COMPLETE'
         stack['generic'].action = 'CREATE'
         stack['generic'].status = 'COMPLETE'
-        info = api.format_stack_outputs(stack, stack.outputs)
+        info = api.format_stack_outputs(stack.outputs)
         expected = [{'description': 'No description given',
                      'output_key': 'incorrect_output'},
                     {'description': 'Good output',
@@ -1053,7 +1053,8 @@ class FormatSoftwareConfigDeploymentTest(common.HeatTestCase):
         self.assertEqual([{'name': 'result'}], result['outputs'])
         self.assertEqual([{'name': 'result'}], result['outputs'])
         self.assertEqual({}, result['options'])
-        self.assertEqual(self.now.isoformat(), result['creation_time'])
+        self.assertEqual(heat_timeutils.isotime(self.now),
+                         result['creation_time'])
 
     def test_format_software_config_none(self):
         self.assertIsNone(api.format_software_config(None))
@@ -1070,8 +1071,10 @@ class FormatSoftwareConfigDeploymentTest(common.HeatTestCase):
         self.assertEqual(deployment.action, result['action'])
         self.assertEqual(deployment.status, result['status'])
         self.assertEqual(deployment.status_reason, result['status_reason'])
-        self.assertEqual(self.now.isoformat(), result['creation_time'])
-        self.assertEqual(self.now.isoformat(), result['updated_time'])
+        self.assertEqual(heat_timeutils.isotime(self.now),
+                         result['creation_time'])
+        self.assertEqual(heat_timeutils.isotime(self.now),
+                         result['updated_time'])
 
     def test_format_software_deployment_none(self):
         self.assertIsNone(api.format_software_deployment(None))
