@@ -11,12 +11,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import datetime
 import mock
 from oslo_config import cfg
 import oslo_db.exception
 
-from heat.common import exception
 from heat.engine import event
 from heat.engine import rsrc_defn
 from heat.engine import stack
@@ -94,7 +92,7 @@ class EventCommon(common.HeatTestCase):
         self.stack.store()
 
         self.resource = self.stack['EventTestResource']
-        self.resource._store()
+        self.resource.store()
         self.addCleanup(stack_object.Stack.delete, self.ctx, self.stack.id)
 
 
@@ -104,76 +102,9 @@ class EventTest(EventCommon):
         super(EventTest, self).setUp()
         self._setup_stack(tmpl)
 
-    def test_load(self):
-        self.resource.resource_id_set('resource_physical_id')
-
-        e = event.Event(self.ctx, self.stack, 'TEST', 'IN_PROGRESS', 'Testing',
-                        'wibble', self.resource.properties,
-                        self.resource.name, self.resource.type())
-
-        e.store()
-        self.assertIsNotNone(e.id)
-
-        loaded_e = event.Event.load(self.ctx, e.id)
-
-        self.assertEqual(self.stack.id, loaded_e.stack.id)
-        self.assertEqual(self.resource.name, loaded_e.resource_name)
-        self.assertEqual('wibble', loaded_e.physical_resource_id)
-        self.assertEqual('TEST', loaded_e.action)
-        self.assertEqual('IN_PROGRESS', loaded_e.status)
-        self.assertEqual('Testing', loaded_e.reason)
-        self.assertIsNotNone(loaded_e.timestamp)
-        self.assertEqual({'Foo': 'goo'}, loaded_e.resource_properties)
-
-    def test_load_with_timestamp(self):
-        self.resource.resource_id_set('resource_physical_id')
-        timestamp = datetime.datetime.utcnow()
-
-        e = event.Event(self.ctx, self.stack, 'TEST', 'IN_PROGRESS', 'Testing',
-                        'wibble', self.resource.properties,
-                        self.resource.name, self.resource.type(),
-                        timestamp=timestamp)
-
-        e.store()
-        self.assertIsNotNone(e.id)
-
-        loaded_e = event.Event.load(self.ctx, e.id)
-
-        self.assertEqual(timestamp, loaded_e.timestamp)
-
-    def test_load_no_event(self):
-        with mock.patch("heat.objects.event.Event") as event_mock:
-            event_mock.get_by_id.return_value = None
-            self.assertRaisesRegex(exception.NotFound,
-                                   "^No event exists with id",
-                                   event.Event.load, self.ctx, 1)
-
-    def test_load_given_stack_event(self):
-        self.resource.resource_id_set('resource_physical_id')
-
-        e = event.Event(self.ctx, self.stack, 'TEST', 'IN_PROGRESS', 'Testing',
-                        'wibble', self.resource.properties,
-                        self.resource.name, self.resource.type())
-
-        e.store()
-        self.assertIsNotNone(e.id)
-
-        ev = event_object.Event.get_by_id(self.ctx, e.id)
-
-        loaded_e = event.Event.load(self.ctx, e.id, stack=self.stack, event=ev)
-
-        self.assertEqual(self.stack.id, loaded_e.stack.id)
-        self.assertEqual(self.resource.name, loaded_e.resource_name)
-        self.assertEqual('wibble', loaded_e.physical_resource_id)
-        self.assertEqual('TEST', loaded_e.action)
-        self.assertEqual('IN_PROGRESS', loaded_e.status)
-        self.assertEqual('Testing', loaded_e.reason)
-        self.assertIsNotNone(loaded_e.timestamp)
-        self.assertEqual({'Foo': 'goo'}, loaded_e.resource_properties)
-
     def test_store_caps_events(self):
-        cfg.CONF.set_override('event_purge_batch_size', 1)
-        cfg.CONF.set_override('max_events_per_stack', 1)
+        cfg.CONF.set_override('event_purge_batch_size', 1, enforce_type=True)
+        cfg.CONF.set_override('max_events_per_stack', 1, enforce_type=True)
         self.resource.resource_id_set('resource_physical_id')
 
         e = event.Event(self.ctx, self.stack, 'TEST', 'IN_PROGRESS', 'Testing',
@@ -212,8 +143,9 @@ class EventTest(EventCommon):
                         'wibble', self.resource.properties,
                         self.resource.name, self.resource.type())
 
-        e.store()
         self.assertIsNone(e.identifier())
+        e.store()
+        self.assertIsNotNone(e.identifier())
 
     def test_badprop(self):
         rname = 'bad_resource'
@@ -226,6 +158,28 @@ class EventTest(EventCommon):
                         'wibble', res.properties, res.name, res.type())
         self.assertIn('Error', e.resource_properties)
 
+    def test_as_dict(self):
+        e = event.Event(self.ctx, self.stack, 'TEST', 'IN_PROGRESS', 'Testing',
+                        'wibble', self.resource.properties,
+                        self.resource.name, self.resource.type())
+
+        e.store()
+        expected = {
+            'id': e.uuid,
+            'timestamp': e.timestamp.isoformat(),
+            'type': 'os.heat.event',
+            'version': '0.1',
+            'payload': {'physical_resource_id': 'wibble',
+                        'resource_action': 'TEST',
+                        'resource_name': 'EventTestResource',
+                        'resource_properties': {'Foo': 'goo'},
+                        'resource_status': 'IN_PROGRESS',
+                        'resource_status_reason': 'Testing',
+                        'resource_type': 'ResourceWithRequiredProps',
+                        'stack_id': self.stack.id,
+                        'version': '0.1'}}
+        self.assertEqual(expected, e.as_dict())
+
 
 class EventTestSingleLargeProp(EventCommon):
 
