@@ -55,6 +55,10 @@ empty_template = template_format.parse('''{
   "HeatTemplateFormatVersion" : "2012-12-12",
 }''')
 
+aws_empty_template = template_format.parse('''{
+  "AWSTemplateFormatVersion" : "2010-09-09",
+}''')
+
 parameter_template = template_format.parse('''{
   "HeatTemplateFormatVersion" : "2012-12-12",
   "Parameters" : {
@@ -115,20 +119,21 @@ class TemplatePluginFixture(fixtures.Fixture):
 class TestTemplatePluginManager(common.HeatTestCase):
     def test_template_NEW_good(self):
         class NewTemplate(template.Template):
-            SECTIONS = (VERSION, MAPPINGS) = ('NEWTemplateFormatVersion',
-                                              '__undefined__')
+            SECTIONS = (VERSION, MAPPINGS, CONDITIONS, PARAMETERS) = (
+                'NEWTemplateFormatVersion',
+                '__undefined__',
+                'conditions',
+                'parameters')
             RESOURCES = 'thingies'
 
-            def param_schemata(self):
+            def param_schemata(self, param_defaults=None):
                 pass
 
             def get_section_name(self, section):
                 pass
 
-            def parameters(self, stack_identifier, user_params):
-                pass
-
-            def validate_resource_definitions(self, stack):
+            def parameters(self, stack_identifier, user_params,
+                           param_defaults=None):
                 pass
 
             def resource_definitions(self, stack):
@@ -140,9 +145,6 @@ class TestTemplatePluginManager(common.HeatTestCase):
             def __getitem__(self, section):
                 return {}
 
-            def functions(self):
-                return {}
-
         class NewTemplatePrint(function.Function):
             def result(self):
                 return 'always this'
@@ -270,6 +272,210 @@ class ParserTest(common.HeatTestCase):
         self.assertIsNot(raw, parsed)
 
 
+class TestTemplateConditionParser(common.HeatTestCase):
+
+    def setUp(self):
+        super(TestTemplateConditionParser, self).setUp()
+        self.ctx = utils.dummy_context()
+        t = {
+            'heat_template_version': '2016-10-14',
+            'parameters': {
+                'env_type': {
+                    'type': 'string',
+                    'default': 'test'
+                }
+            },
+            'conditions': {
+                'prod_env': {
+                    'equals': [{'get_param': 'env_type'}, 'prod']}},
+            'resources': {
+                'r1': {
+                    'type': 'GenericResourceType',
+                    'condition': 'prod_env'
+                }
+            },
+            'outputs': {
+                'foo': {
+                    'condition': 'prod_env',
+                    'value': 'show me'
+                }
+            }
+        }
+
+        self.tmpl = template.Template(t)
+
+    def test_conditions_with_non_supported_functions(self):
+        t = {
+            'heat_template_version': '2016-10-14',
+            'parameters': {
+                'env_type': {
+                    'type': 'string',
+                    'default': 'test'
+                }
+            },
+            'conditions': {
+                'prod_env': {
+                    'equals': [{'get_param': 'env_type'},
+                               {'get_attr': [None, 'att']}]}}}
+        # test with get_attr in equals
+        tmpl = template.Template(t)
+        stk = stack.Stack(self.ctx, 'test_condition_with_get_attr_func', tmpl)
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               tmpl.conditions, stk)
+        self.assertIn('"get_attr" is invalid', six.text_type(ex))
+        self.assertIn('conditions.prod_env.equals[1].get_attr',
+                      six.text_type(ex))
+
+        # test with get_resource in top level of a condition
+        tmpl.t['conditions']['prod_env'] = {'get_resource': 'R1'}
+        stk = stack.Stack(self.ctx, 'test_condition_with_get_attr_func', tmpl)
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               tmpl.conditions, stk)
+        self.assertIn('"get_resource" is invalid', six.text_type(ex))
+
+        # test with get_attr in top level of a condition
+        tmpl.t['conditions']['prod_env'] = {'get_attr': [None, 'att']}
+        stk = stack.Stack(self.ctx, 'test_condition_with_get_attr_func', tmpl)
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               tmpl.conditions, stk)
+        self.assertIn('"get_attr" is invalid', six.text_type(ex))
+
+    def test_condition_resolved_not_boolean(self):
+        t = {
+            'heat_template_version': '2016-10-14',
+            'parameters': {
+                'env_type': {
+                    'type': 'string',
+                    'default': 'test'
+                }
+            },
+            'conditions': {
+                'prod_env': {'get_param': 'env_type'}}}
+
+        # test with get_attr in equals
+        tmpl = template.Template(t)
+        stk = stack.Stack(self.ctx, 'test_condition_not_boolean', tmpl)
+
+        conditions = tmpl.conditions(stk)
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               conditions.is_enabled, 'prod_env')
+        self.assertIn('The definition of condition "prod_env" is invalid',
+                      six.text_type(ex))
+
+    def test_condition_reference_condition(self):
+        t = {
+            'heat_template_version': '2016-10-14',
+            'parameters': {
+                'env_type': {
+                    'type': 'string',
+                    'default': 'test'
+                }
+            },
+            'conditions': {
+                'prod_env': {'equals': [{'get_param': 'env_type'}, 'prod']},
+                'test_env': {'not': 'prod_env'},
+                'prod_or_test_env': {'or': ['prod_env', 'test_env']},
+                'prod_and_test_env': {'and': ['prod_env', 'test_env']},
+            }}
+
+        # test with get_attr in equals
+        tmpl = template.Template(t)
+        stk = stack.Stack(self.ctx, 'test_condition_reference', tmpl)
+        conditions = tmpl.conditions(stk)
+
+        self.assertFalse(conditions.is_enabled('prod_env'))
+        self.assertTrue(conditions.is_enabled('test_env'))
+        self.assertTrue(conditions.is_enabled('prod_or_test_env'))
+        self.assertFalse(conditions.is_enabled('prod_and_test_env'))
+
+    def test_get_res_condition_invalid(self):
+        tmpl = copy.deepcopy(self.tmpl)
+        # test condition name is invalid
+        stk = stack.Stack(self.ctx, 'test_res_invalid_condition', tmpl)
+
+        conds = tmpl.conditions(stk)
+        ex = self.assertRaises(ValueError, conds.is_enabled, 'invalid_cd')
+        self.assertIn('Invalid condition "invalid_cd"', six.text_type(ex))
+        # test condition name is not string
+        ex = self.assertRaises(ValueError, conds.is_enabled, 111)
+        self.assertIn('Invalid condition "111"', six.text_type(ex))
+
+    def test_res_condition_using_boolean(self):
+        tmpl = copy.deepcopy(self.tmpl)
+        # test condition name is boolean
+        stk = stack.Stack(self.ctx, 'test_res_cd_boolean', tmpl)
+
+        conds = tmpl.conditions(stk)
+        self.assertTrue(conds.is_enabled(True))
+        self.assertFalse(conds.is_enabled(False))
+
+    def test_parse_output_condition_invalid(self):
+        stk = stack.Stack(self.ctx,
+                          'test_output_invalid_condition',
+                          self.tmpl)
+
+        # test condition name is invalid
+        self.tmpl.t['outputs']['foo']['condition'] = 'invalid_cd'
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               lambda: stk.outputs)
+        self.assertIn('Invalid condition "invalid_cd"', six.text_type(ex))
+        self.assertIn('outputs.foo.condition', six.text_type(ex))
+        # test condition name is not string
+        self.tmpl.t['outputs']['foo']['condition'] = 222
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               lambda: stk.outputs)
+        self.assertIn('Invalid condition "222"', six.text_type(ex))
+        self.assertIn('outputs.foo.condition', six.text_type(ex))
+
+    def test_conditions_circular_ref(self):
+        t = {
+            'heat_template_version': '2016-10-14',
+            'parameters': {
+                'env_type': {
+                    'type': 'string',
+                    'default': 'test'
+                }
+            },
+            'conditions': {
+                'first_cond': {'not': 'second_cond'},
+                'second_cond': {'not': 'third_cond'},
+                'third_cond': {'not': 'first_cond'},
+            }
+        }
+        tmpl = template.Template(t)
+        stk = stack.Stack(self.ctx, 'test_condition_circular_ref', tmpl)
+        conds = tmpl.conditions(stk)
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               conds.is_enabled, 'first_cond')
+        self.assertIn('Circular definition for condition "first_cond"',
+                      six.text_type(ex))
+
+    def test_parse_output_condition_boolean(self):
+        t = copy.deepcopy(self.tmpl.t)
+        t['outputs']['foo']['condition'] = True
+        stk = stack.Stack(self.ctx,
+                          'test_output_cd_boolean',
+                          template.Template(t))
+
+        self.assertEqual('show me', stk.outputs['foo'].get_value())
+
+        t = copy.deepcopy(self.tmpl.t)
+        t['outputs']['foo']['condition'] = False
+        stk = stack.Stack(self.ctx,
+                          'test_output_cd_boolean',
+                          template.Template(t))
+        self.assertIsNone(stk.outputs['foo'].get_value())
+
+    def test_parse_output_condition_function(self):
+        t = copy.deepcopy(self.tmpl.t)
+        t['outputs']['foo']['condition'] = {'not': 'prod_env'}
+        stk = stack.Stack(self.ctx,
+                          'test_output_cd_function',
+                          template.Template(t))
+
+        self.assertEqual('show me', stk.outputs['foo'].get_value())
+
+
 class TestTemplateValidate(common.HeatTestCase):
 
     def test_template_validate_cfn_check_t_digest(self):
@@ -359,6 +565,72 @@ class TestTemplateValidate(common.HeatTestCase):
         err = tmpl.validate()
         self.assertIsNone(err)
 
+    def test_get_resources_good(self):
+        """Test get resources successful."""
+
+        t = template_format.parse('''
+            AWSTemplateFormatVersion: 2010-09-09
+            Resources:
+              resource1:
+                Type: AWS::EC2::Instance
+                Properties:
+                  property1: value1
+                Metadata:
+                  foo: bar
+                DependsOn: dummy
+                DeletionPolicy: dummy
+                UpdatePolicy:
+                  foo: bar
+        ''')
+
+        expected = {'resource1': {'Type': 'AWS::EC2::Instance',
+                                  'Properties': {'property1': 'value1'},
+                                  'Metadata': {'foo': 'bar'},
+                                  'DependsOn': 'dummy',
+                                  'DeletionPolicy': 'dummy',
+                                  'UpdatePolicy': {'foo': 'bar'}}}
+
+        tmpl = template.Template(t)
+        self.assertEqual(expected, tmpl[tmpl.RESOURCES])
+
+    def test_get_resources_bad_no_data(self):
+        """Test get resources without any mapping."""
+
+        t = template_format.parse('''
+            AWSTemplateFormatVersion: 2010-09-09
+            Resources:
+              resource1:
+        ''')
+
+        tmpl = template.Template(t)
+        error = self.assertRaises(exception.StackValidationFailed,
+                                  tmpl.validate)
+        self.assertEqual('Each Resource must contain a Type key.',
+                         six.text_type(error))
+
+    def test_get_resources_no_type(self):
+        """Test get resources with invalid key."""
+
+        t = template_format.parse('''
+            AWSTemplateFormatVersion: 2010-09-09
+            Resources:
+              resource1:
+                Properties:
+                  property1: value1
+                Metadata:
+                  foo: bar
+                DependsOn: dummy
+                DeletionPolicy: dummy
+                UpdatePolicy:
+                  foo: bar
+        ''')
+
+        tmpl = template.Template(t)
+        error = self.assertRaises(exception.StackValidationFailed,
+                                  tmpl.validate)
+        self.assertEqual('Each Resource must contain a Type key.',
+                         six.text_type(error))
+
     def test_template_validate_hot_check_t_digest(self):
         t = {
             'heat_template_version': '2015-04-30',
@@ -425,6 +697,10 @@ class TemplateTest(common.HeatTestCase):
     def resolve(snippet, template, stack=None):
         return function.resolve(template.parse(stack, snippet))
 
+    @staticmethod
+    def resolve_condition(snippet, template, stack=None):
+        return function.resolve(template.parse_condition(stack, snippet))
+
     def test_defaults(self):
         empty = template.Template(empty_template)
         self.assertNotIn('AWSTemplateFormatVersion', empty)
@@ -451,7 +727,8 @@ class TemplateTest(common.HeatTestCase):
         init_ex = self.assertRaises(exception.InvalidTemplateVersion,
                                     template.Template, invalid_hot_version_tmp)
         valid_versions = ['2013-05-23', '2014-10-16',
-                          '2015-04-30', '2015-10-15', '2016-04-08']
+                          '2015-04-30', '2015-10-15', '2016-04-08',
+                          '2016-10-14', 'newton']
         ex_error_msg = ('The template version is invalid: '
                         '"heat_template_version: 2012-12-12". '
                         '"heat_template_version" should be one of: %s'
@@ -596,7 +873,8 @@ class TemplateTest(common.HeatTestCase):
                  {'Fn::FindInMap': ["ReallyShortList"]})
 
         for find in finds:
-            self.assertRaises(KeyError, self.resolve, find, tmpl, stk)
+            self.assertRaises(exception.StackValidationFailed,
+                              self.resolve, find, tmpl, stk)
 
     def test_param_refs(self):
         env = environment.Environment({'foo': 'bar', 'blarg': 'wibble'})
@@ -696,6 +974,220 @@ class TemplateTest(common.HeatTestCase):
         data = {"Fn::Select": ["one", '']}
         self.assertEqual("", self.resolve(data, tmpl))
 
+    def test_equals(self):
+        tpl = template_format.parse('''
+        AWSTemplateFormatVersion: 2010-09-09
+        Parameters:
+          env_type:
+            Type: String
+            Default: 'test'
+        ''')
+        snippet = {'Fn::Equals': [{'Ref': 'env_type'}, 'prod']}
+        # when param 'env_type' is 'test', equals function resolve to false
+        tmpl = template.Template(tpl)
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_equals_false', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertFalse(resolved)
+        # when param 'env_type' is 'prod', equals function resolve to true
+        tmpl = template.Template(tpl,
+                                 env=environment.Environment(
+                                     {'env_type': 'prod'}))
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_equals_true', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertTrue(resolved)
+
+    def test_equals_invalid_args(self):
+        tmpl = template.Template(aws_empty_template)
+
+        snippet = {'Fn::Equals': ['test', 'prod', 'invalid']}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+
+        error_msg = ('.Fn::Equals: Arguments to "Fn::Equals" must be '
+                     'of the form: [value_1, value_2]')
+        self.assertIn(error_msg, six.text_type(exc))
+        # test invalid type
+        snippet = {'Fn::Equals': {"equal": False}}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+        self.assertIn(error_msg, six.text_type(exc))
+
+    def test_not(self):
+        tpl = template_format.parse('''
+        AWSTemplateFormatVersion: 2010-09-09
+        Parameters:
+          env_type:
+            Type: String
+            Default: 'test'
+        ''')
+        snippet = {'Fn::Not': [{'Fn::Equals': [{'Ref': 'env_type'}, 'prod']}]}
+        # when param 'env_type' is 'test', not function resolve to true
+        tmpl = template.Template(tpl)
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_not_true', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertTrue(resolved)
+        # when param 'env_type' is 'prod', not function resolve to false
+        tmpl = template.Template(tpl,
+                                 env=environment.Environment(
+                                     {'env_type': 'prod'}))
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_not_false', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertFalse(resolved)
+
+    def test_not_invalid_args(self):
+        tmpl = template.Template(aws_empty_template)
+
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_not_invalid', tmpl)
+        snippet = {'Fn::Not': ['invalid_arg']}
+        exc = self.assertRaises(ValueError,
+                                self.resolve_condition, snippet, tmpl, stk)
+
+        error_msg = 'Invalid condition "invalid_arg"'
+        self.assertIn(error_msg, six.text_type(exc))
+        # test invalid type
+        snippet = {'Fn::Not': 'invalid'}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+        error_msg = 'Arguments to "Fn::Not" must be '
+        self.assertIn(error_msg, six.text_type(exc))
+
+        snippet = {'Fn::Not': ['cd1', 'cd2']}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+        error_msg = 'Arguments to "Fn::Not" must be '
+        self.assertIn(error_msg, six.text_type(exc))
+
+    def test_and(self):
+        tpl = template_format.parse('''
+        AWSTemplateFormatVersion: 2010-09-09
+        Parameters:
+          env_type:
+            Type: String
+            Default: 'test'
+          zone:
+            Type: String
+            Default: 'shanghai'
+        ''')
+        snippet = {
+            'Fn::And': [
+                {'Fn::Equals': [{'Ref': 'env_type'}, 'prod']},
+                {'Fn::Not': [{'Fn::Equals': [{'Ref': 'zone'}, "beijing"]}]}]}
+        # when param 'env_type' is 'test', and function resolve to false
+        tmpl = template.Template(tpl)
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_and_false', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertFalse(resolved)
+        # when param 'env_type' is 'prod', and param 'zone' is 'shanghai',
+        # the 'and' function resolve to true
+        tmpl = template.Template(tpl,
+                                 env=environment.Environment(
+                                     {'env_type': 'prod'}))
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_and_true', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertTrue(resolved)
+        # when param 'env_type' is 'prod', and param 'zone' is 'shanghai',
+        # the 'and' function resolve to true
+        tmpl = template.Template(tpl,
+                                 env=environment.Environment(
+                                     {'env_type': 'prod',
+                                      'zone': 'beijing'}))
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_and_false', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertFalse(resolved)
+
+    def test_and_invalid_args(self):
+        tmpl = template.Template(aws_empty_template)
+
+        error_msg = ('The minimum number of condition arguments to "Fn::And" '
+                     'is 2.')
+        snippet = {'Fn::And': ['invalid_arg']}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+        self.assertIn(error_msg, six.text_type(exc))
+
+        error_msg = 'Arguments to "Fn::And" must be'
+        # test invalid type
+        snippet = {'Fn::And': 'invalid'}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+        self.assertIn(error_msg, six.text_type(exc))
+
+        stk = stack.Stack(utils.dummy_context(), 'test_and_invalid', tmpl)
+        snippet = {'Fn::And': ['cd1', True]}
+        exc = self.assertRaises(ValueError,
+                                self.resolve_condition, snippet, tmpl, stk)
+        error_msg = 'Invalid condition "cd1"'
+        self.assertIn(error_msg, six.text_type(exc))
+
+    def test_or(self):
+        tpl = template_format.parse('''
+        AWSTemplateFormatVersion: 2010-09-09
+        Parameters:
+          zone:
+            Type: String
+            Default: 'guangzhou'
+        ''')
+        snippet = {
+            'Fn::Or': [
+                {'Fn::Equals': [{'Ref': 'zone'}, 'shanghai']},
+                {'Fn::Equals': [{'Ref': 'zone'}, 'beijing']}]}
+        # when param 'zone' is neither equal to 'shanghai' nor 'beijing',
+        # the 'or' function resolve to false
+        tmpl = template.Template(tpl)
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_or_false', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertFalse(resolved)
+        # when param 'zone' equals to 'shanghai' or 'beijing',
+        # the 'or' function resolve to true
+        tmpl = template.Template(tpl,
+                                 env=environment.Environment(
+                                     {'zone': 'beijing'}))
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_or_true', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertTrue(resolved)
+
+        tmpl = template.Template(tpl,
+                                 env=environment.Environment(
+                                     {'zone': 'shanghai'}))
+        stk = stack.Stack(utils.dummy_context(),
+                          'test_or_true', tmpl)
+        resolved = self.resolve_condition(snippet, tmpl, stk)
+        self.assertTrue(resolved)
+
+    def test_or_invalid_args(self):
+        tmpl = template.Template(aws_empty_template)
+
+        error_msg = ('The minimum number of condition arguments to "Fn::Or" '
+                     'is 2.')
+        snippet = {'Fn::Or': ['invalid_arg']}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+        self.assertIn(error_msg, six.text_type(exc))
+
+        error_msg = 'Arguments to "Fn::Or" must be'
+        # test invalid type
+        snippet = {'Fn::Or': 'invalid'}
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                self.resolve_condition, snippet, tmpl)
+        self.assertIn(error_msg, six.text_type(exc))
+
+        stk = stack.Stack(utils.dummy_context(), 'test_or_invalid', tmpl)
+        snippet = {'Fn::Or': ['invalid_cd', True]}
+        exc = self.assertRaises(ValueError,
+                                self.resolve_condition, snippet, tmpl, stk)
+        error_msg = 'Invalid condition "invalid_cd"'
+        self.assertIn(error_msg, six.text_type(exc))
+
     def test_join(self):
         tmpl = template.Template(empty_template)
         join = {"Fn::Join": [" ", ["foo", "bar"]]}
@@ -852,9 +1344,9 @@ class TemplateTest(common.HeatTestCase):
         snippet = {'Fn::ResourceFacade': 'wibble'}
         stk = stack.Stack(self.ctx, 'test_stack',
                           template.Template(empty_template))
-        error = self.assertRaises(ValueError,
+        error = self.assertRaises(exception.StackValidationFailed,
                                   self.resolve, snippet, stk.t, stk)
-        self.assertIn(list(six.iterkeys(snippet))[0], six.text_type(error))
+        self.assertIn(next(iter(snippet)), six.text_type(error))
 
     def test_resource_facade_missing_deletion_policy(self):
         snippet = {'Fn::ResourceFacade': 'DeletionPolicy'}
@@ -894,7 +1386,7 @@ class TemplateTest(common.HeatTestCase):
             }
         })
         self.assertEqual(expected_description, tmpl['Description'])
-        self.assertNotIn('Parameters', six.iterkeys(tmpl))
+        self.assertNotIn('Parameters', tmpl.keys())
 
     def test_add_resource(self):
         cfn_tpl = template_format.parse('''
@@ -972,6 +1464,26 @@ class TemplateTest(common.HeatTestCase):
         self.assertEqual({}, empty_template['resources'])
         self.assertEqual({}, empty_template['outputs'])
 
+    def test_create_empty_template_from_another_template(self):
+        res_param_template = template_format.parse('''{
+          "HeatTemplateFormatVersion" : "2012-12-12",
+          "Parameters" : {
+            "foo" : { "Type" : "String" },
+            "blarg" : { "Type" : "String", "Default": "quux" }
+          },
+          "Resources" : {
+            "foo" : { "Type" : "GenericResourceType" },
+            "blarg" : { "Type" : "GenericResourceType" }
+          }
+        }''')
+
+        env = environment.Environment({'foo': 'bar'})
+        hot_tmpl = template.Template(res_param_template, env)
+        empty_template = template.Template.create_empty_template(
+            from_template=hot_tmpl)
+        self.assertEqual({}, empty_template['Resources'])
+        self.assertEqual(hot_tmpl.env, empty_template.env)
+
 
 class TemplateFnErrorTest(common.HeatTestCase):
     scenarios = [
@@ -985,22 +1497,22 @@ class TemplateFnErrorTest(common.HeatTestCase):
          dict(expect=ValueError,
               snippet={"Fn::Select": ["not", "no json"]})),
         ('select_wrong_num_args_1',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Select": []})),
         ('select_wrong_num_args_2',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Select": ["4"]})),
         ('select_wrong_num_args_3',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Select": ["foo", {"foo": "bar"}, ""]})),
         ('select_wrong_num_args_4',
          dict(expect=TypeError,
               snippet={'Fn::Select': [['f'], {'f': 'food'}]})),
         ('split_no_delim',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Split": ["foo, bar, achoo"]})),
         ('split_no_list',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Split": "foo, bar, achoo"})),
         ('base64_list',
          dict(expect=TypeError,
@@ -1014,18 +1526,18 @@ class TemplateFnErrorTest(common.HeatTestCase):
                   {'$var1': 'foo', '%var2%': ['bar']},
                   '$var1 is %var2%']})),
         ('replace_list_mapping',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Replace": [
                   ['var1', 'foo', 'var2', 'bar'],
                   '$var1 is ${var2}']})),
         ('replace_dict',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Replace": {}})),
         ('replace_missing_template',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Replace": [['var1', 'foo', 'var2', 'bar']]})),
         ('replace_none_template',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Replace": [['var2', 'bar'], None]})),
         ('replace_list_string',
          dict(expect=TypeError,
@@ -1039,46 +1551,46 @@ class TemplateFnErrorTest(common.HeatTestCase):
          dict(expect=TypeError,
               snippet={"Fn::Join": [" ", {"foo": "bar"}]})),
         ('join_wrong_num_args_1',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": []})),
         ('join_wrong_num_args_2',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": [" "]})),
         ('join_wrong_num_args_3',
-         dict(expect=ValueError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": [" ", {"foo": "bar"}, ""]})),
         ('join_string_nodelim',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": "o"})),
         ('join_string_nodelim_1',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": "oh"})),
         ('join_string_nodelim_2',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": "ohh"})),
         ('join_dict_nodelim1',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": {"foo": "bar"}})),
         ('join_dict_nodelim2',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": {"foo": "bar", "blarg": "wibble"}})),
         ('join_dict_nodelim3',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::Join": {"foo": "bar", "blarg": "wibble",
                                     "baz": "quux"}})),
         ('member_list2map_no_key_or_val',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::MemberListToMap": [
                   'Key', ['.member.2.Key=metric',
                           '.member.2.Value=cpu',
                           '.member.5.Key=size',
                           '.member.5.Value=56']]})),
         ('member_list2map_no_list',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::MemberListToMap": [
                   'Key', '.member.2.Key=metric']})),
         ('member_list2map_not_string',
-         dict(expect=TypeError,
+         dict(expect=exception.StackValidationFailed,
               snippet={"Fn::MemberListToMap": [
                   'Name', ['Value'], ['.member.0.Name=metric',
                                       '.member.0.Value=cpu',
@@ -1095,8 +1607,7 @@ class TemplateFnErrorTest(common.HeatTestCase):
         error = self.assertRaises(self.expect,
                                   resolve,
                                   self.snippet)
-        self.assertIn(list(six.iterkeys(self.snippet))[0],
-                      six.text_type(error))
+        self.assertIn(next(iter(self.snippet)), six.text_type(error))
 
 
 class ResolveDataTest(common.HeatTestCase):
